#include "BumpMap.h"

BumpMap::BumpMap()
{
	m_bumpMap = nullptr;
	m_bumpMapWidth = 0;
	m_bumpMapHeight = 0;
}

bool BumpMap::Initialize(char* filename)
{
	bool result;

	result = LoadBumpMap(filename);
	
	return result;
}

bool BumpMap::LoadBumpMap(char* filename)
{
	FILE* filePtr;
	int error;
	unsigned int count;
	BITMAPFILEHEADER bitmapFileHeader;
	BITMAPINFOHEADER bitmapInfoHeader;
	int imageSize, k, index;
	unsigned char* bitmapImage;
	int bitmapWidth, bitmapHeight;

	//Open the file in binary
	error = fopen_s(&filePtr, filename, "rb");
	if (error != 0)
	{
		return false;
	}

	//Read in the file header
	count = fread(&bitmapFileHeader, sizeof(BITMAPFILEHEADER), 1, filePtr);
	if (count != 1)
	{
		return false;
	}

	//Read in the info header
	count = fread(&bitmapInfoHeader, sizeof(BITMAPINFOHEADER), 1, filePtr);
	if (count != 1)
	{
		return false;
	}

	m_bumpMapWidth = bitmapInfoHeader.biWidth;
	m_bumpMapHeight = bitmapInfoHeader.biHeight;

	imageSize = m_bumpMapWidth * m_bumpMapHeight * 3;

	bitmapImage = new unsigned char[imageSize];
	if (!bitmapImage)
	{
		return false;
	}

	//Move to the beginning of the image data
	fseek(filePtr, bitmapFileHeader.bfOffBits, SEEK_SET);

	//Read in the bitmap data
	count = fread(bitmapImage, 1, imageSize, filePtr);
	if (count != imageSize)
	{
		return false;
	}

	error = fclose(filePtr);
	if (error != 0)
	{
		return false;
	}

	//Create the bump map storage
	m_bumpMap = new Vector3[m_bumpMapWidth * m_bumpMapHeight];
	if (!m_bumpMap)
	{
		return false;
	}

	for (int i = 0; i < imageSize; i+=3)
	{
		m_bumpMap[i / 3].x = bitmapImage[i] / 255.0f;
		m_bumpMap[i / 3].y = bitmapImage[i + 1] / 255.0f;
		m_bumpMap[i / 3].z = bitmapImage[i + 2] / 255.0f;
	}

	//delete the stored bitmap now that its done with
	delete[] bitmapImage;
	bitmapImage = nullptr;

	return true;
}

Vector3 BumpMap::SampleBumpMap(double tu, double tv)
{
	Vector3 output;
	int index;
	int sampleX, sampleY;

	if (tu > 1.0f)
	{
		tu = tu - floor(tu);
	}
	else if (tu < 0.0f)
	{
		tu = abs(tu) - (floor(abs(tu)));
	}

	if (tv > 1.0f)
	{
		tv = tv - floor(tv);
	}
	else if (tu < 0.0f)
	{
		tv = abs(tv) - (floor(abs(tv)));
	}

	sampleX = m_bumpMapWidth * tu;
	sampleY = m_bumpMapHeight * tv;

	index = (m_bumpMapHeight * sampleX) + (sampleY);

	output = m_bumpMap[index];


	output.x = (output.x * 2.0f) - 1.0f;
	output.y = (output.y * 2.0f) - 1.0f;
	output.z = (output.z * 2.0f) - 1.0f;

	return output;
}

void BumpMap::Shutdown()
{
	if (m_bumpMap)
	{
		delete[] m_bumpMap;
		m_bumpMap = nullptr;
	}
}