#include "Camera.h"

CameraClass::CameraClass()
{
	position = Vector3(0.0f, 0.0f, 30.0f);
	rotation = Vector3(0.0f, 0.0f, 0.0f);
	lookAt = Vector3(0.0f, 0.0f, 0.0f);
	up = Vector3(0.0f, 1.0f, 0.0f);
	right = Vector3(0.0f, 0.0f, 0.0f);

	reflectPosition = Vector3(0.0f, 0.0f, 0.0f);
	reflectUp = Vector3(0.0f, 0.0f, 0.0f);
	reflectLook = Vector3(0.0f, 0.0f, 0.0f);

	m_trackDistance = Vector3(0.0f, 0.0f, 0.0f);
	m_trackingTarget = nullptr;
}

CameraClass::CameraClass(const CameraClass& other)
{
}

CameraClass::~CameraClass()
{
}

void CameraClass::Initialize(float start_x, float start_y, float start_z)
{
	m_cameraType = FP_CAMERA;
	SetPosition(Vector3(start_x, start_y, start_z));
}

void CameraClass::Initialize(Vector3* tracker, float dist_x, float dist_y, float dist_z)
{
	m_cameraType = TRACKING_CAMERA;
	m_trackingTarget = tracker;
	m_trackDistance = Vector3(dist_x, dist_y, dist_z);
	CalculateAngles();
}

void CameraClass::CalculateAngles()
{
	rotation.x = -atan2(m_trackDistance.y, m_trackDistance.z) * (180.0f/3.14f);
	rotation.y = -atan2(m_trackDistance.z, m_trackDistance.x) * (180.0f / 3.14f);
}

void CameraClass::Frame(Input* input_, float dt_)
{
	dt = dt_;

	if (m_cameraType == FP_CAMERA)
	{
		if (input_->isKeyDown('W'))
		{
			MoveForward();
		}
		else if (input_->isKeyDown('S'))
		{
			MoveBackward();
		}

		if (input_->isKeyDown('A'))
		{
			MoveLeft();
		}
		else if (input_->isKeyDown('D'))
		{
			MoveRight();
		}

		if (input_->getMouseX() > input_->getMousePrevX())
		{
			rotation.y += 0.1f * (dt * 1000);
			input_->setMouseX(input_->getMouseX());
		}
		else if (input_->getMouseX() < input_->getMousePrevX())
		{
			rotation.y -= 0.1f * (dt * 1000);
			input_->setMouseX(input_->getMouseX());
		}

		if (input_->getMouseY() < input_->getMousePrevY())
		{
			rotation.x += 0.1f * (dt * 1000);
			input_->setMouseY(input_->getMouseY());
		}
		else if (input_->getMouseY() > input_->getMousePrevY())
		{
			rotation.x -= 0.1f * (dt * 1000);
			input_->setMouseY(input_->getMouseY());
		}

		if (input_->isKeyDown(VK_LEFT))
		{
			rotation.y -= 0.1f * (dt * 1000);
		}
		else if (input_->isKeyDown(VK_RIGHT))
		{
			rotation.y += 0.1f * (dt * 1000);
		}

		if (input_->isKeyDown(VK_DOWN))
		{
			rotation.x -= 0.1f * (dt * 1000);
		}
		else if (input_->isKeyDown(VK_UP))
		{
			rotation.x += 0.1f * (dt * 1000);
		}

		EnforceLocking();

		Update();
	}
	else if (m_cameraType == TRACKING_CAMERA)
	{
		Vector3 _position = *m_trackingTarget + m_trackDistance;
		if (!moveLockX)
		{
			position.x = _position.x;
		}

		if (!moveLockY)
		{
			position.y = _position.y;
		}

		if (!moveLockZ)
		{
			position.z = _position.z;
		}

		EnforceLocking();

		Update();
	}

}

void CameraClass::Frame()
{
	if (m_cameraType == TRACKING_CAMERA)
	{
		Vector3 _position = *m_trackingTarget + m_trackDistance;
		if (!moveLockX)
		{
			position.x = _position.x;
		}

		if (!moveLockY)
		{
			position.y = _position.y;
		}

		if (!moveLockZ)
		{
			position.z = _position.z;
		}

		EnforceLocking();
		
		Update();
	}
}

void CameraClass::EnforceLocking()
{
	if (moveLockX)
	{
		position.x = xMoveLockValue;
	}

	if (moveLockY)
	{
		position.y = yMoveLockValue;
	}

	if (moveLockZ)
	{
		position.z = zMoveLockValue;
	}

	if (rotLockX)
	{
		rotation.x = xRotLockValue;
	}

	if (rotLockY)
	{
		rotation.y = yRotLockValue;
	}

	if (rotLockZ)
	{
		rotation.z = zRotLockValue;
	}

}

void CameraClass::Update()
{
	float cosR, cosP, cosY;
	float sinR, sinP, sinY;

	//These values are always needed to set variables to save time
	cosY = cosf(rotation.y*3.1415 / 180);
	cosP = cosf(rotation.x*3.1415 / 180);
	cosR = cosf(rotation.z*3.1415 / 180);
	sinY = sinf(rotation.y*3.1415 / 180);
	sinP = sinf(rotation.x*3.1415 / 180);
	sinR = sinf(rotation.z*3.1415 / 180);

	//Calculate the forward vector
	forward.x = sinY * cosP;
	forward.y = sinP;
	forward.z = cosP * -cosY;

	//Calculate the lookAt vector
	lookAt.x = forward.x + position.x;
	lookAt.y = forward.y + position.y;
	lookAt.z = forward.z + position.z;

	//Calculate the up vector
	up.x = -cosY * sinR - sinY * sinP * cosR;
	up.y = cosP * cosR;
	up.z = -sinY * sinR - sinP * cosR * -cosY;

	//Calculate the right vector
	right = forward.cross(up);

}

void CameraClass::TurnX(int mouseDiff)
{
	rotation.x += 0.01f * mouseDiff;

	if (rotation.x >= 360.0f)
	{
		rotation.x -= 360.0f;
	}
	else if (rotation.x < 0.0f)
	{
		rotation.x += 360.0f;
	}
}

void CameraClass::TurnY(int mouseDiff)
{
	rotation.y += 0.01f * mouseDiff;

	if (rotation.y >= 360.0f)
	{
		rotation.y -= 360.0f;
	}
	else if (rotation.z < 0.0f)
	{
		rotation.y += 360.0f;
	}
}

void CameraClass::MoveForward()
{
	float moveX, moveY, moveZ;
	float radianX, radianY;

	//Convert Y and X rotation to radians
	radianX = (((360 - rotation.x) * 3.1415) / 180.0f);
	radianY = ((rotation.y * 3.1415) / 180.0f);

	//Calculate the amount to move in x, y and z based on the current rotation
	moveX = sin(radianY) *  (10.0f * dt);

	moveY = -tan(radianX) * (10.0f * dt);

	moveZ = -cos(radianY) * (10.0f * dt);

	//Move in x, y and z axis the required amount to move 1 unit in the correct vector

	position.x += moveX;

	position.y += moveY;

	position.z += moveZ;

}

void CameraClass::MoveBackward()
{
	float moveX, moveY, moveZ;
	float radianX, radianY;

	//Convert rotation to radians
	radianX = (((360 - rotation.x) * 3.1415) / 180.0f);
	radianY = ((rotation.y * 3.1415) / 180.0f);

	//Calculate the amount to move based on the current rotation
	moveX = -sin(radianY) * (10.0f * dt);

	moveY = tan(radianX) * (10.0f * dt);

	moveZ = cos(radianY) * (10.0f * dt);

	//Move in x, y and z the required amount to move 1 unit in the vector

	position.x += moveX;

	position.y += moveY;

	position.z += moveZ;
}

void CameraClass::MoveLeft()
{
	float moveX, moveZ;
	float radianY;

	//Temporarily convert Y and X rotation to radians
	radianY = (((rotation.y - 90) * 3.1415) / 180.0f);

	//Calculate the amount to move in x and z based on the current camera angle
	moveX = sin(radianY) * (10.0f * dt);

	moveZ = -cos(radianY) * (10.0f * dt);

	//Move in x and z axis the required amount to move 1 unit in the vector

	position.x += moveX;

	position.z += moveZ;
}

void CameraClass::MoveRight()
{
	float moveX, moveZ;
	float radianY;

	//Temporarily convert Y and X rotation to radians
	radianY = (((rotation.y + 90) * 3.1415) / 180.0f);

	//Calculate the amount to move in x and z based on the current camera angle
	moveX = sin(radianY) * (10.0f * dt);

	moveZ = -cos(radianY) * (10.0f * dt);

	//Move in x and z axis the required amount to move 1 unit in the vector

	position.x += moveX;

	position.z += moveZ;

}

void CameraClass::RenderReflection(float height)
{
	float radians;

	reflectUp.x = 0.0f;
	reflectUp.y = 1.0f;
	reflectUp.z = 0.0f;

	reflectPosition.x = position.x;
	reflectPosition.y = -position.y + (height * 2.0f);
	reflectPosition.z = position.z;

	radians = rotation.y * 0.0174532925f;

	reflectLook.x = sinf(radians) + position.x;
	reflectLook.y = position.y;
	reflectLook.z = cosf(radians) + position.z;
}

void CameraClass::RenderRefraction(Vector3 objectPos)
{
	float cosR, cosP, cosY;
	float sinR, sinP, sinY;

	//These values are always needed to set variables to save time
	cosY = cosf(rotation.y*3.1415 / 180);
	cosP = cosf(rotation.x*3.1415 / 180);
	cosR = cosf(rotation.z*3.1415 / 180);
	sinY = sinf(rotation.y*3.1415 / 180);
	sinP = sinf(rotation.x*3.1415 / 180);
	sinR = sinf(rotation.z*3.1415 / 180);

	//Calculate the lookAt vector
	refractLook.x = forward.x + objectPos.x;
	refractLook.y = forward.y + objectPos.y;
	refractLook.z = forward.z + objectPos.z;

	//Calculate the up vector
	refractUp.x = -cosY * sinR - sinY * sinP * cosR;
	refractUp.y = cosP * cosR;
	refractUp.z = -sinY * sinR - sinP * cosR * -cosY;
}