#include "CameraViewModel.h"

bool CameraViewModel::Initialize(CameraClass* renderingCamera, float viewWidth, float viewHeight, float posX, float posY)
{
	bool result;

	m_Camera = renderingCamera;
	m_Wireframe = false;

	m_viewWidth = viewWidth;
	m_viewHeight = viewHeight;

	result = InitializeRenderBuffer();
	if (!result)
	{
		return false;
	}

	result = InitializeModel();
	if (!result)
	{
		return false;
	}

	SetTranslation(posX, posY, 1.0f);

	return true;
}

bool CameraViewModel::InitializeModel()
{
	m_vertexCount = 6;

	m_vertices = new VertexType[m_vertexCount];
	if (!m_vertices)
	{
		return false;
	}

	//Setup a basic square to render the minimap on
	//Triangle One
	m_vertices[0].x = -0.5f;
	m_vertices[0].y = 0.5f;
	m_vertices[0].z = 0.0f;
	m_vertices[0].tu = 0.0f;
	m_vertices[0].tv = 1.0f;

	m_vertices[1].x = 0.5f;
	m_vertices[1].y = 0.5f;
	m_vertices[1].z = 0.0f;
	m_vertices[1].tu = 1.0f;
	m_vertices[1].tv = 1.0f;

	m_vertices[2].x = 0.5f;
	m_vertices[2].y = -0.5f;
	m_vertices[2].z = 0.0f;
	m_vertices[2].tu = 1.0f;
	m_vertices[2].tv = 0.0f;

	//Triangle Two
	m_vertices[3].x = -0.5f;
	m_vertices[3].y = 0.5f;
	m_vertices[3].z = 0.0f;
	m_vertices[3].tu = 0.0f;
	m_vertices[3].tv = 1.0f;

	m_vertices[4].x = 0.5f;
	m_vertices[4].y = -0.5f;
	m_vertices[4].z = 0.0f;
	m_vertices[4].tu = 1.0f;
	m_vertices[4].tv = 0.0f;

	m_vertices[5].x = -0.5f;
	m_vertices[5].y = -0.5f;
	m_vertices[5].z = 0.0f;
	m_vertices[5].tu = 0.0f;
	m_vertices[5].tv = 0.0f;

	return true;
}
bool CameraViewModel::InitializeRenderBuffer()
{
	m_drawBufferName = 0;
	glGenFramebuffers(1, &m_drawBufferName);
	glBindFramebuffer(GL_FRAMEBUFFER, m_drawBufferName);

	glGenTextures(1, &m_texture);

	glBindTexture(GL_TEXTURE_2D, m_texture);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, m_viewWidth, m_viewHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

	glGenRenderbuffers(1, &m_drawDepthBuffer);
	glBindRenderbuffer(GL_RENDERBUFFER, m_drawDepthBuffer);
	glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT, m_viewWidth, m_viewHeight);
	glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, m_drawDepthBuffer);

	glFramebufferTexture(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT1, m_texture, 0);

	m_drawBuffers[0] = GL_COLOR_ATTACHMENT1;
	glDrawBuffers(1, m_drawBuffers);

	if (glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
		return false;
}

void CameraViewModel::Frame()
{

}

void CameraViewModel::Render()
{
	glBindFramebuffer(GL_FRAMEBUFFER, 0);

	glBindTexture(GL_TEXTURE_2D, GetTexture());
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	float x, y, z;

	glPushMatrix();
	GetTranslation(x, y, z);
	glTranslatef(x, y, z);
	GetScaling(x, y, z);
	glScalef(x, y, z);
	GetRotation(x, y, z);
	glRotatef(z, 0, 0, 1);
	glRotatef(y, 0, 1, 0);
	glRotatef(x, 1, 0, 0);

	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH);
	glDisable(GL_FOG);

	glBegin(GL_TRIANGLES);

	glColor3f(0.0f, 0.0f, 0.0f);
	glVertex3f(-0.525f, 0.525f, 0.0f);
	glVertex3f(0.525f, 0.525f, 0.0f);
	glVertex3f(0.525f, -0.525f, 0.0f);
	glVertex3f(-0.525f, 0.525f, 0.0f);
	glVertex3f(0.525f, -0.525f, 0.0f);
	glVertex3f(-0.525f, -0.525f, 0.0f);
	glColor3f(1.0f, 1.0f, 1.0f);

	for (int i = 0; i < m_vertexCount; i++)
	{
		glTexCoord2f(m_vertices[i].tu, m_vertices[i].tv);
		glVertex3f(m_vertices[i].x, m_vertices[i].y, m_vertices[i].z);
	}

	glEnd();
	glPopMatrix();

	glEnable(GL_LIGHTING);
	glEnable(GL_DEPTH);
	glEnable(GL_FOG);
}

CameraClass* CameraViewModel::GetCamera()
{
	return m_Camera;
}

GLuint CameraViewModel::GetDrawBuffer()
{
	return m_drawBufferName;
}
