#include "FireModel.h"

bool FireModel::Initialize(char* fireTexture, char* noiseTexture, char* alphaTexture)
{
	bool result;

	LoadTexture(fireTexture);

	m_noiseTextureB = SOIL_load_OGL_texture(noiseTexture, SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT);

	m_alphaTexture = SOIL_load_OGL_texture(alphaTexture, SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT);
	
	m_MVP = glm::mat4(1.0f);

	glGenVertexArrays(1, &m_vao);
	glBindVertexArray(m_vao);

	GLfloat vertices[] = {
		-1.0f, 1.0f, 0.0f, 0.0f, 0.0f, -1.0f, 0.0f, 0.0f,
		1.0f, 1.0f, 0.0f, 0.0f, 0.0f, -1.0f, 1.0f, 0.0f,
		1.0f, -1.0f, 0.0f, 0.0f, 0.0f, -1.0f, 1.0f, 1.0f,
		-1.0f, -1.0f, 0.0f, 0.0f, 0.0f, -1.0f, 0.0f, 1.0f
	};
	glGenBuffers(1, &m_vbo);

	glBindBuffer(GL_ARRAY_BUFFER, m_vbo);
	glBufferData(GL_ARRAY_BUFFER, sizeof(m_vertices), m_vertices, GL_STATIC_DRAW);

	glGenBuffers(1, &m_ebo);

	GLuint elements[] = {
		0, 1, 2,
		0, 2, 3 
	};

	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, m_ebo);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, sizeof(elements), elements, GL_STATIC_DRAW);

	GLuint vertexShader = glCreateShader(GL_VERTEX_SHADER);
	glShaderSource(vertexShader, 1, &vertexSource, NULL);
	glCompileShader(vertexShader);

	GLuint fragmentShader = glCreateShader(GL_FRAGMENT_SHADER);
	glShaderSource(fragmentShader, 1 ,&fragmentSource, NULL);
	glCompileShader(fragmentShader);

	m_shaderProgram = glCreateProgram();
	glAttachShader(m_shaderProgram, vertexShader);
	glAttachShader(m_shaderProgram, fragmentShader);
	glBindFragDataLocation(m_shaderProgram, 0, "outColor");
	glLinkProgram(m_shaderProgram);
	glUseProgram(m_shaderProgram);

	m_posAttrib = glGetAttribLocation(m_shaderProgram, "position");
	glEnableVertexAttribArray(m_posAttrib);
	glVertexAttribPointer(m_posAttrib, 3, GL_FLOAT, GL_FALSE, 8 * sizeof(GLfloat), 0);

	m_normalAttrib = glGetAttribLocation(m_shaderProgram, "normal");
	glEnableVertexAttribArray(m_normalAttrib);
	glVertexAttribPointer(m_normalAttrib, 3, GL_FLOAT, GL_FALSE, 8 * sizeof(GLfloat), (void*)(3 * sizeof(GLfloat)));

	m_texAttrib = glGetAttribLocation(m_shaderProgram, "texcoord");
	glEnableVertexAttribArray(m_texAttrib);
	glVertexAttribPointer(m_texAttrib, 2, GL_FLOAT, GL_FALSE, 8 * sizeof(GLfloat), (void*)(6 * sizeof(GLfloat)));

	uniMVP = glGetUniformLocation(m_shaderProgram, "MVP");
	uniFrameTime = glGetUniformLocation(m_shaderProgram, "frameTime");
	uniScrollSpeeds = glGetUniformLocation(m_shaderProgram, "scrollSpeeds");
	uniScales = glGetUniformLocation(m_shaderProgram, "scales");

	uniDistortion1 = glGetUniformLocation(m_shaderProgram, "distortion1");
	uniDistortion2 = glGetUniformLocation(m_shaderProgram, "distortion2");
	uniDistortion3 = glGetUniformLocation(m_shaderProgram, "distortion3");
	uniDistortionScale = glGetUniformLocation(m_shaderProgram, "distortionScale");
	uniDistortionBias = glGetUniformLocation(m_shaderProgram, "distortionBias");

	return true;
}

void FireModel::UpdateMVP(glm::mat4 &projection, glm::mat4 &view)
{
	CalculateTransformMatrix();

	glm::mat4 world = 
	{
		GetTransformMatrixValue(0, 0), GetTransformMatrixValue(1, 0), GetTransformMatrixValue(2, 0), 1.0f,
		GetTransformMatrixValue(0, 1), GetTransformMatrixValue(1, 1), GetTransformMatrixValue(2, 1), 1.0f,
		GetTransformMatrixValue(0, 2), GetTransformMatrixValue(1, 2), GetTransformMatrixValue(2, 2), 1.0f,
		GetTransformMatrixValue(0, 3), GetTransformMatrixValue(1, 3), GetTransformMatrixValue(2, 3), 1.0f
	};

	m_MVP = projection * view * world;
}

void FireModel::Frame(float dt)
{

}

void FireModel::Render()
{
	float sampleLocation[2];
	Vector3 perturbValue, finalNoise, noise1, noise2, noise3;
	float noiseCoords[2], noiseCoordsA[2], noiseCoordsB[2], noiseCoordsC[2];
	float perturb;

	float x, y, z;

	glPushMatrix();
	GetScaling(x, y, z);
	glScalef(x, y, z);
	GetTranslation(x, y, z);
	glTranslatef(x, y, z);
	GetRotation(x, y, z);
	glRotatef(z, 0, 0, 1);
	glRotatef(y, 0, 1, 0);
	glRotatef(x, 1, 0, 0);

	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, m_texture);
	glUniform1i(glGetUniformLocation(m_shaderProgram, "texFire"), 0);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, m_alphaTexture);
	glUniform1i(glGetUniformLocation(m_shaderProgram, "texAlpha"), 1);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	glActiveTexture(GL_TEXTURE2);
	glBindTexture(GL_TEXTURE_2D, m_noiseTextureB);
	glUniform1i(glGetUniformLocation(m_shaderProgram, "texNoise"), 2);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	glUniform1f(uniFrameTime, m_frameTime);
	glUniform3f(uniScrollSpeeds, m_scrollSpeeds.x, m_scrollSpeeds.y, m_scrollSpeeds.z);
	glUniform3f(uniScales, m_scales.x, m_scales.y, m_scales.z);
	
	glUniform2f(uniDistortion1, 0.1f, 0.2f);
	glUniform2f(uniDistortion2, 0.1f, 0.3f);
	glUniform2f(uniDistortion3, 0.1f, 0.1f);
	glUniform1f(uniDistortionScale, 0.8f);
	glUniform1f(uniDistortionBias, 0.5f);

	glUniformMatrix4fv(uniMVP, 1, GL_FALSE, &m_MVP[0][0]);

	glDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_INT, 0);

	/*
	glBindTexture(GL_TEXTURE_2D, m_texture);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	BlendTextureAlpha(m_texture, m_alphaTexture, 1.0f);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);

	glBegin(GL_TRIANGLES);

	for (int i = 0; i < m_vertexCount; i++)
	{
		noiseCoordsA[0] = (m_vertices[i].tv * m_scale.x);
		noiseCoordsA[1] = (m_vertices[i].tv * m_scale.x) + (m_frameTime * m_scrollSpeeds.x);

		noiseCoordsB[0] = (m_vertices[i].tu * m_scale.y);
		noiseCoordsB[1] = (m_vertices[i].tv * m_scale.y) + (m_frameTime * m_scrollSpeeds.y);

		noiseCoordsC[0] = (m_vertices[i].tu * m_scale.z);
		noiseCoordsC[1] = (m_vertices[i].tv * m_scale.z) + (m_frameTime * m_scrollSpeeds.y);

		noise1 = m_noiseTexture->SampleBumpMap(noiseCoordsA[0], noiseCoordsA[1]);
		noise2 = m_noiseTexture->SampleBumpMap(noiseCoordsB[0], noiseCoordsB[1]);
		noise3 = m_noiseTexture->SampleBumpMap(noiseCoordsC[0], noiseCoordsC[1]);

		perturb = ((1.0f - m_vertices[i].tu) + m_distortionScale) * m_distortionBias;

		noiseCoords[0] = (finalNoise.x * perturb) + m_vertices[i].tu;
		noiseCoords[1] = (finalNoise.y * perturb) + m_vertices[i].tv;

		glMultiTexCoord2f(GL_TEXTURE0, noiseCoords[0], noiseCoords[1]);
		glMultiTexCoord2f(GL_TEXTURE1, noiseCoords[0], noiseCoords[1]);

		glVertex3f(m_vertices[i].x, m_vertices[i].y, m_vertices[i].z);
	}

	glEnd();
	glPopMatrix();

	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	glDisable(GL_BLEND);

	CleanUpTexEnv();
	*/
}

void FireModel::Shutdown()
{

}