#ifndef _FIREMODEL_H_
#define _FIREMODEL_H_

#include <gl/glm.hpp>

#include "Model.h"
#include "TextureCombiner.h"

class FireModel : public Model
{
public:
	bool Initialize(char* fireTexture, char* noiseTexture, char* alphaTexture);
	void Frame(float dt);
	void Render();
	void Shutdown();

	void UpdateMVP(glm::mat4 &projection, glm::mat4 &view);

private:
	bool InitializeModel();

private:
	glm::mat4 m_MVP;
	GLuint m_alphaTexture;
	GLuint m_noiseTextureB;
	Vector3 m_scrollSpeeds;
	Vector3 m_scales;
	float m_frameTime;
	float m_distortionScale;
	float m_distortionBias;
	GLuint m_vao;
	GLuint m_vbo;
	GLuint m_ebo;
	GLint m_posAttrib;
	GLint m_normalAttrib;
	GLint m_texAttrib;
	GLuint m_shaderProgram;
	GLint uniMVP, uniFrameTime, uniScrollSpeeds, uniScales;
	GLint uniDistortion1, uniDistortion2, uniDistortion3, uniDistortionScale, uniDistortionBias;
	const GLchar* vertexSource =
		"#version 150 core\n"
		"in vec3 position;"
		"in vec3 normal;"
		"in vec2 texcoord;"
		"out vec3 Color;"
		"out vec2 Texcoord1;"
		"out vec2 Texcoord2;"
		"out vec2 Texcoord3;"
		"uniform mat4 MVP;"
		"uniform float frameTime;"
		"uniform vec3 scrollSpeeds;"
		"uniform vec3 scales;"
		"void main() {"
		"	Normal = normal;"
		"	Texcoords1 = texcoord * scales.x;"
		"	Texcoords1.y = Texcoords1.y + (frameTime * scrollSpeeds.x);"
		"	Texcoords2 = texcoord * scales.y;"
		"	Texcoords2.y = Texcoords2.y + (frameTime * scrollSpeeds.y);"
		"	Texcoords3 = texcoord * scales.y;"
		"	Texcoords3.y = Texcoords3.y + (frameTime * scrollSpeeds.z);"
		"	gl_Position = MVP * vec4(position, 1.0);"
		"}";
	const GLchar* fragmentSource =
		"#version 150 core\n"
		"in vec3 Normal;"
		"in vec2 Texcoord1;"
		"in vec2 Texcoord2;"
		"in vec2 Texcoord3;"
		"out vec4 outColor;"
		"uniform sampler2D texFire;"
		"uniform sampler2D texNoise;"
		"uniform sampler2D texAlpha;"
		"uniform vec2 distortion1;"
		"uniform vec2 distortion2;"
		"uniform vec2 distortion3;"
		"uniform float distortionScale;"
		"uniform float distortionBias;"
		"void main() {"
		"	float4 noise1;"
		"	float4 noise2;"
		"	float4 noise3;"
		"	float4 finalNoise;"
		"	float perturb;"
		"	float2 noiseCoords;"
		"	float4 fireColor;"
		"	float4 alphaColor;"
		"	noise1  = texture(texNoise, Texcoord1);"
		"	noise2 = texture(texNoise, Texcoord2);"
		"	noise3 = texture(texNoise, Texcoord3);"
		"	noise1.xy = noise1.xy * distortion1.xy;"
		"	noise2.xy = noise2.xy * distortion2.xy;"
		"	noise3.xy = noise3.xy * distortion3.xy;"
		"	finalNoise = noise1 + noise2 + noise3;"
		"	perturb = ((1.0f - input.tex.y) * distortionScale) + distortionBias;"
		"	noiseCoords.xy = (finalNoise.xy * perturb) + input.tex.xy;"
		"	fireColor = texture(texFire, noiseCoords.xy);"
		"	alphaColor = texture(texAlpha, noiseCoords.xy);"
		"	fireColor.a = alphaColor;"
		"	outColor = fireColor;"
		"}";
};

#endif