#include "H_Model.h"

void HModel::Initialize(Font* font, char* text)
{
	num_models = strlen(text);

	m_models = new FontModel[num_models];
	m_rotLocks = new bool[num_models];

	for (int i = 0; i < num_models; i++)
	{
		char* character = new char[2];
		character[0] = text[i];
		character[1] = '\0';
		if (i == 0)
		{
			m_models[i].Initialize(font, character, 10.0f, 30.0f, 1.0f, 1.0f, 1.0f);
		}
		else
		{
			m_models[i].Initialize(font, character, 10.0f, 0.0f, 1.0f, 1.0f, 1.0f);
		}

		m_rotLocks[i] = false;
	}
}

void HModel::Frame(Input* input, float dt)
{
	rotY = (dt*20.0f) * rotFlipper;

	if (input->isKeyDown(VK_NUMPAD4))
	{
		input->SetKeyUp(VK_NUMPAD4);
		if (model_num > 0)
			model_num--;
	}

	if (input->isKeyDown(VK_NUMPAD6))
	{
		input->SetKeyUp(VK_NUMPAD6);
		if (model_num < num_models)
			model_num++;
	}

	if (input->isKeyDown(VK_NUMPAD5))
	{
		input->SetKeyUp(VK_NUMPAD5);
		m_rotLocks[model_num] = !m_rotLocks[model_num];
	}

	if (input->isKeyDown(VK_NUMPAD8))
	{
		input->SetKeyUp(VK_NUMPAD8);
		float x, y, z;
		m_models[model_num].GetTranslation(x, y, z);
		m_models[model_num].SetTranslation(x + 10.0f, y, z);
	}

	if (input->isKeyDown(VK_NUMPAD2))
	{
		input->SetKeyUp(VK_NUMPAD2);
		float x, y, z;
		m_models[model_num].GetTranslation(x, y, z);
		m_models[model_num].SetTranslation(x - 10.0f, y, z);
	}

	if (input->isKeyDown(VK_NUMPAD0))
	{
		input->SetKeyUp(VK_NUMPAD0);
		rotFlipper = -rotFlipper;
	}

	for (int i = 0; i < num_models; i++)
	{
		if (!m_rotLocks[i])
		{
			float rx, ry, rz;
			m_models[i].GetRotation(rx, ry, rz);
			m_models[i].SetRotation(0.0f, ry + rotY, 0.0f);
		}
		m_models[i].CalculateTransformMatrix();
	}
}

void HModel::Render()
{
	for (int i = 0; i < num_models; i++)
	{
		if (m_rotLocks[i])
			glColor3f(1.0f, 0.0f, 0.0f);

		if (i == model_num)
			glColor3f(0.0f, 1.0f, 0.0f);

		m_models[i].RenderHierarchy();

		glColor3f(1.0f, 1.0f, 1.0f);
	}

	for (int i = 0; i < num_models; i++)
	{
		glPopMatrix();
	}
}