#include "Model.h"

#include "TerrainModel.h"

Model::Model()
{
	m_vertices = nullptr;
	m_indices = nullptr;
	m_bumpMap = nullptr;
	srand(time(NULL));
	isIndexed = false;

	m_translate = Vector3(0.0f, 0.0f, 0.0f);
	m_rotate = Vector3(0.0f, 0.0f, 0.0f);
	m_scale = Vector3(1.0f, 1.0f, 1.0f);
}

Model::Model(const Model& other)
{
}

Model::~Model()
{
}

void Model::Render()
{
	glBindTexture(GL_TEXTURE_2D, GetTexture());
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	float x, y, z;

	glPushMatrix();
	GetTranslation(x, y, z);
	glTranslatef(x, y, z);
	GetScaling(x, y, z);
	glScalef(x, y, z);
	GetRotation(x, y, z);
	glRotatef(z, 0, 0, 1);
	glRotatef(y, 0, 1, 0);
	glRotatef(x, 1, 0, 0);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glBegin(GL_TRIANGLES);

	for (int i = 0; i < m_vertexCount; i++)
	{
		glNormal3f(m_vertices[i].nx, m_vertices[i].ny, m_vertices[i].nz);
		glTexCoord2f(m_vertices[i].tu, m_vertices[i].tv);
		glVertex3f(m_vertices[i].x, m_vertices[i].y, m_vertices[i].z);
	}

	glEnd();
	glDisable(GL_BLEND);
	glPopMatrix();
}

void Model::Shutdown()
{
	if (m_vertices)
	{
		delete[] m_vertices;
		m_vertices = nullptr;
	}

	if (m_indices)
	{
		delete[] m_indices;
		m_indices = nullptr;
	}
}

void Model::CalculateTransformMatrix()
{
	float x, y, z;
	x = m_translate.x;
	y = m_translate.y;
	z = m_translate.z;

	float TranslationMatrix[4][4] =
	{
		{ 1, 0, 0, 0 },
		{ 0, 1, 0, 0 },
		{ 0, 0, 1, 0 },
		{ x, y, z, 1 }
	};

	x = m_scale.x;
	y = m_scale.y;
	z = m_scale.z;

	float ScalingMatrix[4][4] =
	{
		{ x, 0, 0, 0 },
		{ 0, y, 0, 0 },
		{ 0, 0, z, 0 },
		{ 0, 0, 0, 1 }
	};

	x = (m_rotate.x * M_PI) / 180;
	y = (m_rotate.y * M_PI) / 180;
	z = (m_rotate.z * M_PI) / 180;

	float XRotationMatrix[4][4] =
	{
		{ 1, 0, 0, 0 },
		{ 0, cos(x), sin(x), 0 },
		{ 0, -sin(x), cos(x), 0 },
		{ 0, 0, 0, 1 }
	};

	float YRotationMatrix[4][4] =
	{
		{ cos(y), 0, -sin(y), 0 },
		{ 0, 1, 0, 0 },
		{ sin(y), 0, cos(y), 0 },
		{ 0, 0, 0, 1 }

	};

	float ZRotationMatrix[4][4] =
	{
		{ cos(z), sin(z), 0, 0 },
		{ -sin(z), cos(z), 0, 0 },
		{ 0, 0, 1, 0 },
		{ 0, 0, 0, 1 }
	};

	float RotationMatrix[4][4];

	MatrixMultiply(RotationMatrix, XRotationMatrix, YRotationMatrix);
	MatrixMultiply(RotationMatrix, RotationMatrix, ZRotationMatrix);

	MatrixMultiply(TransformationMatrix, ScalingMatrix, RotationMatrix);
	MatrixMultiply(TransformationMatrix, TransformationMatrix, TranslationMatrix);

}

void Model::MatrixMultiply(float output[4][4], float matrixA[4][4], float matrixB[4][4])
{
	float outputMatrix[4][4];

	for (int y = 0; y < 4; y++)
	{
		for (int x = 0; x < 4; x++)
		{
			outputMatrix[y][x] = (matrixA[y][0] * matrixB[0][x]) + (matrixA[y][1] * matrixB[1][x]) + (matrixA[y][2] * matrixB[2][x]) + (matrixA[y][3] * matrixB[3][x]);
		}
	}

	for (int y = 0; y < 4; y++)
	{
		for (int x = 0; x < 4; x++)
		{
			output[y][x] = outputMatrix[y][x];
		}
	}
}

void Model::RenderPlanarShadow(float light_pos[4])
{
	const int numThreads = 16;

	GLfloat v0[3], v1[3], v2[3];

	// To define a plane that matches the floor, we need to 3 vertices from it
	float aX, aY, aZ;

	v0[0] = 20.0f;
	v0[1] = 10.0f;
	v0[2] = 0.0f;

	v1[0] = 20.0f;
	v1[1] = 10.0f;
	v1[2] = 1.0f;

	v2[0] = 21.0f;
	v2[1] =	10.0f;
	v2[2] = 0.0f;

	// Build a shadow matrix using the light's current position and the plane
	GenerateShadowMatrix(m_shadowMatrix, light_pos, v0, v1, v2);

	float x, y, z;

	glDisable(GL_DEPTH_TEST);
	glDisable(GL_FOG);
	glDisable(GL_LIGHTING);
	glDisable(GL_TEXTURE_2D);

	glPushMatrix();
		glMultMatrixf((GLfloat *)m_shadowMatrix);
		//translate to floor and draw shadow, remember to match any transforms on the object
		GetTranslation(x, y, z);
		glTranslatef(20.0f, 10.0f, 0.0f);
		GetRotation(x, y, z);
		glRotatef(z, 0, 0, 1);
		glRotatef(y, 0, 1, 0);
		glRotatef(x, 1, 0, 0);
		GetScaling(x, y, z);
		glScalef(x, y, z);
		glBegin(GL_TRIANGLES);
			glColor3f(0.1f, 0.1f, 0.1f); // Shadow's color
	
	
				for (int i = 0; i < m_vertexCount; i++)
				{
					glVertex3f(m_vertices[i].x, m_vertices[i].y, m_vertices[i].z);
				}

		glEnd();
	glPopMatrix();
	glColor3f(1.0f, 1.0f, 1.0f);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_FOG);
	glEnable(GL_TEXTURE_2D);

	
}

void Model::CreateShadowImposter(void* terrainPointer)
{
	float x, y, z;
	float posX, posY, posZ;
	
	m_shadowVertices = new VertexType[16];

	int index = 0;

	TerrainModel* terrain = reinterpret_cast<TerrainModel*>(terrainPointer);

	GetTranslation(x, y, z);

	//Draw the top left quad
	posX = x - (2.0f * m_scale.x);
	posZ = z - (2.0f * m_scale.z);
	terrain->GetHeightAtPosition(posX, posZ, posY);
	m_shadowVertices[index].x = posX;
	m_shadowVertices[index].y = posY;
	m_shadowVertices[index].z = posZ;
	m_shadowVertices[index].tu = 0.0f;
	m_shadowVertices[index].tv = 0.0f;
	index++;

	posX = x;
	posZ = z - (2.0f * m_scale.z);
	terrain->GetHeightAtPosition(posX, posZ, posY);
	m_shadowVertices[index].x = posX;
	m_shadowVertices[index].y = posY;
	m_shadowVertices[index].z = posZ;
	m_shadowVertices[index].tu = 0.5f;
	m_shadowVertices[index].tv = 0.0f;
	index++;
	
	posX = x;
	posZ = z;
	terrain->GetHeightAtPosition(posX, posZ, posY);
	m_shadowVertices[index].x = posX;
	m_shadowVertices[index].y = posY;
	m_shadowVertices[index].z = posZ;
	m_shadowVertices[index].tu = 0.5f;
	m_shadowVertices[index].tv = 0.5f;
	index++;

	posX = x - (2.0f * m_scale.x);
	posZ = z;
	terrain->GetHeightAtPosition(posX, posZ, posY);
	m_shadowVertices[index].x = posX;
	m_shadowVertices[index].y = posY;
	m_shadowVertices[index].z = posZ;
	m_shadowVertices[index].tu = 0.0f;
	m_shadowVertices[index].tv = 0.5f;
	index++;
	
	//Draw the top right quad
	posX = x;
	posZ = z - (2.0f * m_scale.z);
	terrain->GetHeightAtPosition(posX, posZ, posY);
	m_shadowVertices[index].x = posX;
	m_shadowVertices[index].y = posY;
	m_shadowVertices[index].z = posZ;
	m_shadowVertices[index].tu = 0.5f;
	m_shadowVertices[index].tv = 0.0f;
	index++;
	
	posX = x + (2.0f * m_scale.x);
	posZ = z - (2.0f * m_scale.z);
	terrain->GetHeightAtPosition(posX, posZ, posY);
	m_shadowVertices[index].x = posX;
	m_shadowVertices[index].y = posY;
	m_shadowVertices[index].z = posZ;
	m_shadowVertices[index].tu = 1.0f;
	m_shadowVertices[index].tv = 0.0f;
	index++;
	
	posX = x + (2.0f * m_scale.x);
	posZ = z;
	terrain->GetHeightAtPosition(posX, posZ, posY);
	m_shadowVertices[index].x = posX;
	m_shadowVertices[index].y = posY;
	m_shadowVertices[index].z = posZ;
	m_shadowVertices[index].tu = 1.0f;
	m_shadowVertices[index].tv = 0.5f;
	index++;
	
	posX = x;
	posZ = z;
	terrain->GetHeightAtPosition(posX, posZ, posY);
	m_shadowVertices[index].x = posX;
	m_shadowVertices[index].y = posY;
	m_shadowVertices[index].z = posZ;
	m_shadowVertices[index].tu = 0.5f;
	m_shadowVertices[index].tv = 0.5f;
	index++;
	
	//Draw the bottom right quad
	posX = x;
	posZ = z;
	terrain->GetHeightAtPosition(posX, posZ, posY);
	m_shadowVertices[index].x = posX;
	m_shadowVertices[index].y = posY;
	m_shadowVertices[index].z = posZ;
	m_shadowVertices[index].tu = 0.5f;
	m_shadowVertices[index].tv = 0.5f;
	index++;
	
	posX = x + (2.0f * m_scale.x);
	posZ = z;
	terrain->GetHeightAtPosition(posX, posZ, posY);
	m_shadowVertices[index].x = posX;
	m_shadowVertices[index].y = posY;
	m_shadowVertices[index].z = posZ;
	m_shadowVertices[index].tu = 1.0f;
	m_shadowVertices[index].tv = 0.5f;
	index++;
	
	posX = x + (2.0f * m_scale.x);
	posZ = z + (2.0f * m_scale.z);
	terrain->GetHeightAtPosition(posX, posZ, posY);
	m_shadowVertices[index].x = posX;
	m_shadowVertices[index].y = posY;
	m_shadowVertices[index].z = posZ;
	m_shadowVertices[index].tu = 1.0f;
	m_shadowVertices[index].tv = 1.0f;
	index++;

	posX = x;
	posZ = z + (2.0f * m_scale.z);
	terrain->GetHeightAtPosition(posX, posZ, posY);
	m_shadowVertices[index].x = posX;
	m_shadowVertices[index].y = posY;
	m_shadowVertices[index].z = posZ;
	m_shadowVertices[index].tu = 0.5f;
	m_shadowVertices[index].tv = 1.0f;
	index++;
	
	//Draw the bottom left quad
	posX = x - (2.0f * m_scale.x);
	posZ = z;
	terrain->GetHeightAtPosition(posX, posZ, posY);
	m_shadowVertices[index].x = posX;
	m_shadowVertices[index].y = posY;
	m_shadowVertices[index].z = posZ;
	m_shadowVertices[index].tu = 0.0f;
	m_shadowVertices[index].tv = 0.5f;
	index++;

	posX = x;
	posZ = z;
	terrain->GetHeightAtPosition(posX, posZ, posY);
	m_shadowVertices[index].x = posX;
	m_shadowVertices[index].y = posY;
	m_shadowVertices[index].z = posZ;
	m_shadowVertices[index].tu = 0.5f;
	m_shadowVertices[index].tv = 0.5f;
	index++;
	
	posX = x;
	posZ = z + (2.0f * m_scale.z);
	terrain->GetHeightAtPosition(posX, posZ, posY);
	m_shadowVertices[index].x = posX;
	m_shadowVertices[index].y = posY;
	m_shadowVertices[index].z = posZ;
	m_shadowVertices[index].tu = 0.5f;
	m_shadowVertices[index].tv = 1.0f;
	index++;
	
	posX = x - (2.0f * m_scale.x);
	posZ = z + (2.0f * m_scale.z);
	terrain->GetHeightAtPosition(posX, posZ, posY);
	m_shadowVertices[index].x = posX;
	m_shadowVertices[index].y = posY;
	m_shadowVertices[index].z = posZ;
	m_shadowVertices[index].tu = 0.0f;
	m_shadowVertices[index].tv = 1.0f;
	index++;
	
}

void Model::RenderShadow()
{
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);
	

	glBindTexture(GL_TEXTURE_2D, m_imposterTexture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	glBegin(GL_QUADS);

	for (int i = 0; i < 16; i++)
	{
		glTexCoord2f(m_shadowVertices[i].tu, m_shadowVertices[i].tv);
		glVertex3f(m_shadowVertices[i].x, m_shadowVertices[i].y, m_shadowVertices[i].z);
	}

	glEnd();

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
}

void Model::GenerateShadowMatrix(float matrix[16], float light_pos[4], GLfloat v0[3], GLfloat v1[3], GLfloat v2[3])
{
	Vector3 P(v0[0], v0[1], v0[2]);
	Vector3 Q(v1[0], v1[1], v1[2]);
	Vector3 R(v2[0], v2[1], v2[2]);

	Vector3 PQ = (Q - P);
	PQ.normalise();
	Vector3 PR = (R - P);
	PR.normalise();
	Vector3 normal = PR.cross(PQ);

	float a, b, c, d;
	a = normal.getX();
	b = normal.getY();
	c = normal.getZ();
	d = normal.dot(P);

	float x, y, z;
	x = light_pos[0];
	y = light_pos[1];
	z = light_pos[2];

	matrix[0] = d - (b * y + c * z);
	matrix[1] = a * y;
	matrix[2] = a * z;
	matrix[3] = a;

	matrix[4] = b * x;
	matrix[5] = d - (a * x + c * z);
	matrix[6] = b * z;
	matrix[7] = b;

	matrix[8] = c * x;
	matrix[9] = c * y;
	matrix[10] = d - (a * x + b * y);
	matrix[11] = c;

	matrix[12] = -d * x;
	matrix[13] = -d * y;
	matrix[14] = -d * z;
	matrix[15] = -(a * x + b * y + c * z);
}

float Model::GetTransformMatrixValue(int x, int y)
{
	return TransformationMatrix[y][x];
}

void Model::GetTranslation(float& x, float& y, float& z)
{
	x = m_translate.x;
	y = m_translate.y;
	z = m_translate.z;
}

void Model::GetRotation(float& x, float& y, float& z)
{
	x = m_rotate.x;
	y = m_rotate.y;
	z = m_rotate.z;
}

void Model::GetScaling(float& x, float& y, float& z)
{
	x = m_scale.x;
	y = m_scale.y;
	z = m_scale.z;
}

void Model::SetTranslation(float x, float y, float z)
{
	m_translate.x = x;
	m_translate.y = y;
	m_translate.z = z;
}

void Model::SetRotation(float x, float y, float z)
{
	m_rotate.x = x;
	m_rotate.y = y;
	m_rotate.z = z;
}

void Model::SetScaling(float x, float y, float z)
{
	m_scale.x = x;
	m_scale.y = y;
	m_scale.z = z;
}

int Model::GetVertexCount()
{
	return m_vertexCount;
}

VertexType Model::GetVertex(int vertexNumber)
{
	return m_vertices[vertexNumber];
}

int Model::GetIndexCount()
{
	return m_indexCount;
}

long Model::GetIndex(int indexNumber)
{
	return m_indices[indexNumber];
}

GLuint Model::GetTexture()
{
	return m_texture;
}

void Model::LoadTexture(char* filename)
{
	m_texture = SOIL_load_OGL_texture(filename, SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT);
}

void Model::SetShadowImposter(GLuint imposter)
{
	m_imposterTexture = imposter;
}

void Model::SetTexture(GLuint texture)
{
	m_texture = texture;
}

void Model::LoadBumpMap(char* filename)
{
	m_bumpMap = new BumpMap;
	if (m_bumpMap)
	{
		m_bumpMap->Initialize(filename);
	}
}

Vector3 Model::SampleBumpMap(float tu, float tv)
{
	return m_bumpMap->SampleBumpMap(tu, tv);
}