#include "ModelObject.h"

ModelObject::ModelObject()
{
	m_Model = nullptr;
	m_translate.x = 0;
	m_translate.y = 0;
	m_translate.z = 0;
	m_rotate.x = 0;
	m_rotate.y = 0;
	m_rotate.z = 0;
	m_scale.x = 1;
	m_scale.y = 1;
	m_scale.z = 1;
	total_dt = 0.0f;
}

ModelObject::ModelObject(const ModelObject& other)
{
}

ModelObject::~ModelObject()
{
}

bool ModelObject::Initialize(char* modelFilename)
{
	bool result;

	m_Model = new TerrainModel;
	if (!m_Model)
	{
		return false;
	}

	//m_Model->Initialize(modelFilename);

	//m_Model->InitCube(4, 2);

	//m_Model->InitLerpCubeSphere();

	//m_Model->InitGeodesicSphere(4);

	//m_Model->InitGeodesicSphereIndexed(4);

	//m_Model->InitializeObj(modelFilename);

	//m_Model->InitIndexedPlane((24*24), 24);

	TerrainModel* terrain = static_cast<TerrainModel*>(m_Model);
	terrain->Initialize(modelFilename);
	terrain = nullptr;

	return true;
	
}

void ModelObject::Shutdown()
{
	if (m_Model)
	{
		m_Model->Shutdown();
		delete m_Model;
		m_Model = nullptr;
	}
}

void ModelObject::Frame(float dt)
{
	//m_rotate.x += 100 * dt;
	//m_rotate.y += 100 * dt;
	//m_rotate.z += 100 * dt;

	//m_Model->LerpCubeSphere(dt);
	total_dt += dt;

	//m_Model->GeodesicSphereStableNoise(dt);

	
	CalculateTransformMatrix();
}

void ModelObject::CalculateTransformMatrix()
{
	float x, y, z;
	x = m_translate.x;
	y = m_translate.y;
	z = m_translate.z;

	float TranslationMatrix[4][4] = 
	{
		{ 1, 0, 0, 0 },
		{ 0, 1, 0, 0 },
		{ 0, 0, 1, 0 },
		{ x, y, z, 1 }
	};

	x = m_scale.x;
	y = m_scale.y;
	z = m_scale.z;

	float ScalingMatrix[4][4] =
	{
		{ x, 0, 0, 0 },
		{ 0, y, 0, 0 },
		{ 0, 0, z, 0 },
		{ 0, 0, 0, 1 }
	};

	x = (m_rotate.x * M_PI)/180;
	y = (m_rotate.y * M_PI)/180;
	z = (m_rotate.z * M_PI)/180;

	float XRotationMatrix[4][4] = 
	{
		{ 1, 0, 0, 0 },
		{ 0, cos(x), sin(x), 0 },
		{ 0, -sin(x), cos(x), 0 },
		{ 0, 0, 0, 1 }
	};

	float YRotationMatrix[4][4] = 
	{
		{ cos(y), 0, -sin(y), 0 },
		{ 0, 1, 0, 0 },
		{ sin(y), 0, cos(y), 0 },
		{ 0, 0, 0, 1 }

	};

	float ZRotationMatrix[4][4] =
	{
		{ cos(z), sin(z), 0, 0 },
		{ -sin(z), cos(z), 0, 0 },
		{ 0, 0, 1, 0 },
		{ 0, 0, 0, 1 }
	};

	float RotationMatrix[4][4];

	MatrixMultiply(RotationMatrix, XRotationMatrix, YRotationMatrix);
	MatrixMultiply(RotationMatrix, RotationMatrix, ZRotationMatrix);

	MatrixMultiply(TransformationMatrix, ScalingMatrix, RotationMatrix);
	MatrixMultiply(TransformationMatrix, TransformationMatrix, TranslationMatrix);

}

void ModelObject::MatrixMultiply(float output[4][4], float matrixA[4][4], float matrixB[4][4])
{
	float outputMatrix[4][4];

	for (int y = 0; y < 4; y++)
	{
		for (int x = 0; x < 4; x++)
		{
			outputMatrix[y][x] = (matrixA[y][0] * matrixB[0][x]) + (matrixA[y][1] * matrixB[1][x]) + (matrixA[y][2] * matrixB[2][x]) + (matrixA[y][3] * matrixB[3][x]);
		}
	}

	for (int y = 0; y < 4; y++)
	{
		for (int x = 0; x < 4; x++)
		{
			output[y][x] = outputMatrix[y][x];
		}
	}
}

Model* ModelObject::GetModel()
{
	return m_Model;
}

float ModelObject::GetTransformMatrixValue(int x, int y)
{
	return TransformationMatrix[y][x];
}

void ModelObject::GetTranslation(float& x, float& y, float& z)
{
	x = m_translate.x;
	y = m_translate.y;
	z = m_translate.z;
}

void ModelObject::GetRotation(float& x, float& y, float& z)
{
	x = m_rotate.x;
	y = m_rotate.y;
	z = m_rotate.z;
}

void ModelObject::GetScaling(float& x, float& y, float& z)
{
	x = m_scale.x;
	y = m_scale.y;
	z = m_scale.z;
}

void ModelObject::SetTranslation(float x, float y, float z)
{
	m_translate.x = x;
	m_translate.y = y;
	m_translate.z = z;
}

void ModelObject::SetRotation(float x, float y, float z)
{
	m_rotate.x = x;
	m_rotate.y = y;
	m_rotate.z = z;
}

void ModelObject::SetScaling(float x, float y, float z)
{
	m_scale.x = x;
	m_scale.y = y;
	m_scale.z = z;
}