#include "ObjMtlModel.h"

bool ObjMtlModel::LoadObj(char* modelFilename)
{
	std::ifstream fileStream;
	int fileSize = 0;

	fileStream.open(modelFilename, std::ifstream::in);

	if (fileStream.is_open() == false)
		return false;

	fileStream.seekg(0, std::ios::end);
	fileSize = (int)fileStream.tellg();
	fileStream.seekg(0, std::ios::beg);

	if (fileSize <= 0)
		return false;

	char *buffer = new char[fileSize];

	if (buffer == 0)
		return false;

	memset(buffer, '\0', fileSize);

	TokenStream tokenStream, lineStream, faceStream, matStream;
	string tempLine, token;

	fileStream.read(buffer, fileSize);
	tokenStream.SetTokenStream(buffer);

	delete[] buffer;

	tokenStream.ResetStream();

	float tempX, tempY, tempZ;
	vector<Vector3> verts, norms, texC;
	vector<int> faces;
	char* tempMat;

	char lineDelimiters[2] = { '\n', ' ' };

	while (tokenStream.MoveToNextLine(&tempLine))
	{
		lineStream.SetTokenStream((char*)tempLine.c_str());
		tokenStream.GetNextToken(0, 0, 0);

		if (!lineStream.GetNextToken(&token, lineDelimiters, 2))
			continue;

		if (strcmp(token.c_str(), "v") == 0)
		{
			lineStream.GetNextToken(&token, lineDelimiters, 2);
			tempX = (float)atof(token.c_str());

			lineStream.GetNextToken(&token, lineDelimiters, 2);
			tempY = (float)atof(token.c_str());

			lineStream.GetNextToken(&token, lineDelimiters, 2);
			tempZ = (float)atof(token.c_str());

			verts.push_back(Vector3(tempX, tempY, tempZ));
		}
		else if (strcmp(token.c_str(), "vn") == 0)
		{
			lineStream.GetNextToken(&token, lineDelimiters, 2);
			tempX = (float)atof(token.c_str());

			lineStream.GetNextToken(&token, lineDelimiters, 2);
			tempY = (float)atof(token.c_str());

			lineStream.GetNextToken(&token, lineDelimiters, 2);
			tempZ = (float)atof(token.c_str());

			norms.push_back(Vector3(tempX, tempY, tempZ));
		}
		else if (strcmp(token.c_str(), "vt") == 0)
		{
			lineStream.GetNextToken(&token, lineDelimiters, 2);
			tempX = (float)atof(token.c_str());

			lineStream.GetNextToken(&token, lineDelimiters, 2);
			tempY = (float)atof(token.c_str());

			texC.push_back(Vector3(tempX, tempY, 0));
		}
		else if (strcmp(token.c_str(), "f") == 0)
		{
			char faceTokens[3] = { '\n', ' ', '/' };
			std::string faceIndex;

			faceStream.SetTokenStream((char*)tempLine.c_str());
			faceStream.GetNextToken(0, 0, 0);

			for (int i = 0; i < 3; i++)
			{
				faceStream.GetNextToken(&faceIndex, faceTokens, 3);
				faces.push_back((int)atoi(faceIndex.c_str()));

				faceStream.GetNextToken(&faceIndex, faceTokens, 3);
				faces.push_back((int)atoi(faceIndex.c_str()));

				faceStream.GetNextToken(&faceIndex, faceTokens, 3);
				faces.push_back((int)atoi(faceIndex.c_str()));
			}
		}
		else if (strcmp(token.c_str(), "#") == 0)
		{

		}
		else if (strcmp(token.c_str(), "mtllib") == 0)
		{
			matStream.SetTokenStream((char*)tempLine.c_str());
			matStream.GetNextToken(0, 0, 0);

		}
		else if (strcmp(token.c_str(), "usemtl") == 0)
		{

		}
	}
}