#include "PrimitiveModels.h"

bool Plane::Initialize(int quadsPerFace, int textureRepeats)
{
	//isIndexed = true;

	//Check that the poly count is a square number
	if (quadsPerFace > 0 && sqrt(quadsPerFace) == floor(sqrt(quadsPerFace)) && textureRepeats > 0)
	{
		int vertex_count = 0;
		float subdivisions = sqrt(quadsPerFace);
		m_vertexCount = pow((subdivisions + 1), 2);
		m_indexCount = quadsPerFace * 6;
		m_vertexCount = m_indexCount;
		m_vertices = new VertexType[m_vertexCount];

		if ((int)subdivisions % textureRepeats != 0)
		{
			textureRepeats = 1;
		}

		//Create top face
		for (int z = 0; z < subdivisions; z++)
		{
			for (int x = 0; x < subdivisions; x++)
			{
				//One quad
				//Triangle One
				m_vertices[vertex_count].x = 0.5f - (x *(1 / subdivisions));
				m_vertices[vertex_count].y = 0;
				m_vertices[vertex_count].z = -0.5f + (z * (1 / subdivisions));
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x) * textureRepeats;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * z) * textureRepeats;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = 0.5f - (x *(1 / subdivisions)) - (1 / subdivisions);
				m_vertices[vertex_count].y = 0;
				m_vertices[vertex_count].z = -0.5f + (z * (1 / subdivisions));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions)) * textureRepeats;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * z) * textureRepeats;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = 0.5f - (x *(1 / subdivisions));
				m_vertices[vertex_count].y = 0;
				m_vertices[vertex_count].z = -0.5f + (z * (1 / subdivisions)) + (1 / subdivisions);
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x) * textureRepeats;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * z) + (1 / subdivisions)) * textureRepeats;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				//Triangle Two
				m_vertices[vertex_count].x = 0.5f - (x *(1 / subdivisions)) - (1 / subdivisions);
				m_vertices[vertex_count].y = 0;
				m_vertices[vertex_count].z = -0.5f + (z * (1 / subdivisions));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions)) * textureRepeats;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * z) * textureRepeats;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = 0.5f - (x *(1 / subdivisions)) - (1 / subdivisions);
				m_vertices[vertex_count].y = 0;
				m_vertices[vertex_count].z = -0.5f + (z * (1 / subdivisions)) + (1 / subdivisions);
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions)) * textureRepeats;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * z) + (1 / subdivisions)) * textureRepeats;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;
					
				m_vertices[vertex_count].x = 0.5f - (x *(1 / subdivisions));
				m_vertices[vertex_count].y = 0;
				m_vertices[vertex_count].z = -0.5f + (z * (1 / subdivisions)) + (1 / subdivisions);
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x) * textureRepeats;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * z) + (1 / subdivisions)) * textureRepeats;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;
			}
		}
	}
	else
	{
		return false;
	}

	return true;
}

bool Plane::InitializeIndexed(int quadsPerFace, int textureRepeats)
{
	float tx, tz;
	float x1, y1, z1;
	int current_vertex_count = 0, current_index_count = 0;

	isIndexed = true;

	//Check that the poly count is a square number
	if (quadsPerFace > 0 && sqrt(quadsPerFace) == floor(sqrt(quadsPerFace)) && textureRepeats > 0)
	{
		bool found = false;
		int vertex_count = 0;
		int vertexNum = 0;
		float subdivisions = sqrt(quadsPerFace);
		m_vertexCount = pow((subdivisions + 1), 2);
		m_indexCount = quadsPerFace * 6;
		m_vertices = new VertexType[m_vertexCount];
		m_indices = new long[m_indexCount];

		if ((int)subdivisions % textureRepeats != 0)
		{
			textureRepeats = 1;
		}

		//Create top face
		for (int z = 0; z < subdivisions; z++)
		{
			for (int x = 0; x < subdivisions; x++)
			{
				//One quad
				//Triangle One
				x1 = 0.5f - (x *(1 / subdivisions));
				y1= 0;
				z1 = -0.5f + (z * (1 / subdivisions));
				tx = ((1 / subdivisions) * x) * textureRepeats;
				tz = ((1 / subdivisions) * z) * textureRepeats;

				AssignVertex(current_vertex_count, current_index_count, subdivisions, textureRepeats, tx, tz, x1, y1, z1);

				x1 = 0.5f - (x *(1 / subdivisions)) - (1 / subdivisions);
				y1 = 0;
				z1 = -0.5f + (z * (1 / subdivisions));
				tx = (((1 / subdivisions) * x) + (1 / subdivisions)) * textureRepeats;
				tz = ((1 / subdivisions) * z) * textureRepeats;

				AssignVertex(current_vertex_count, current_index_count, subdivisions, textureRepeats, tx, tz, x1, y1, z1);

				x1 = 0.5f - (x *(1 / subdivisions));
				y1 = 0;
				z1 = -0.5f + (z * (1 / subdivisions)) + (1 / subdivisions);
				tx = ((1 / subdivisions) * x) * textureRepeats;
				tz = (((1 / subdivisions) * z) + (1 / subdivisions)) * textureRepeats;

				AssignVertex(current_vertex_count, current_index_count, subdivisions, textureRepeats, tx, tz, x1, y1, z1);

				//Triangle Two
				x1 = 0.5f - (x *(1 / subdivisions)) - (1 / subdivisions);
				y1 = 0;
				z1 = -0.5f + (z * (1 / subdivisions));
				tx = (((1 / subdivisions) * x) + (1 / subdivisions)) * textureRepeats;
				tz = ((1 / subdivisions) * z) * textureRepeats;

				AssignVertex(current_vertex_count, current_index_count, subdivisions, textureRepeats, tx, tz, x1, y1, z1);

				x1 = 0.5f - (x *(1 / subdivisions)) - (1 / subdivisions);
				y1 = 0;
				z1 = -0.5f + (z * (1 / subdivisions)) + (1 / subdivisions);
				tx = (((1 / subdivisions) * x) + (1 / subdivisions)) * textureRepeats;
				tz = (((1 / subdivisions) * z) + (1 / subdivisions)) * textureRepeats;

				AssignVertex(current_vertex_count, current_index_count, subdivisions, textureRepeats, tx, tz, x1, y1, z1);

				x1 = 0.5f - (x *(1 / subdivisions));
				y1 = 0;
				z1 = -0.5f + (z * (1 / subdivisions)) + (1 / subdivisions);
				tx = ((1 / subdivisions) * x) * textureRepeats;
				tz = (((1 / subdivisions) * z) + (1 / subdivisions)) * textureRepeats;

				AssignVertex(current_vertex_count, current_index_count, subdivisions, textureRepeats, tx, tz, x1, y1, z1);
			}
		}
	}
	else
	{
		return false;
	}

	return true;
}

void Plane::AssignVertex(int& vertex_count, int& index_count, int subdivisions, int textureRepeats, float tx, float tz, float x, float y, float z)
{
	bool found = false;
	int vertexNum = 0;

	for (int i = 0; i < vertex_count; i++)
	{
		if (x == m_vertices[i].x && y == m_vertices[i].y && z == m_vertices[i].z)
		{
			vertexNum = i;
			found = true;
		}
	}

	if (found)
	{
		m_indices[index_count] = vertexNum;
		index_count++;
	}
	else
	{
		m_vertices[vertex_count].x = x;
		m_vertices[vertex_count].y = y;
		m_vertices[vertex_count].z = z;
		m_vertices[vertex_count].tu = tx;
		m_vertices[vertex_count].tv = tz;
		m_vertices[vertex_count].nx = 0.0f;
		m_vertices[vertex_count].ny = 1.0f;
		m_vertices[vertex_count].nz = 0.0f;
		m_indices[index_count] = vertex_count;
		vertex_count++;
		index_count++;
	}
}

bool Disc::Initialize(int numSegs)
{
	if (numSegs >= 4)
	{
		int vertexNum;
		float angleBetween = 2 * M_PI / numSegs;
		m_vertexCount = numSegs * 3;
		m_vertices = new VertexType[m_vertexCount];

		for (int i = 0; i < numSegs; i++)
		{
			vertexNum = i * 3;
			m_vertices[vertexNum].x = cos(i*angleBetween);
			m_vertices[vertexNum].y = sin(i*angleBetween);
			m_vertices[vertexNum].z = 0.0f;
			m_vertices[vertexNum].nx = 0.0f;
			m_vertices[vertexNum].ny = 0.0f;
			m_vertices[vertexNum].nz = 1.0f;
			m_vertices[vertexNum].tu = (0.5f + (cos(i*angleBetween) / 2));
			m_vertices[vertexNum].tv = (0.5f + (sin(i*angleBetween) / 2));

			vertexNum++;
			m_vertices[vertexNum].x = 0.0f;
			m_vertices[vertexNum].y = 0.0f;
			m_vertices[vertexNum].z = 0.0f;
			m_vertices[vertexNum].nx = 0.0f;
			m_vertices[vertexNum].ny = 0.0f;
			m_vertices[vertexNum].nz = 1.0f;
			m_vertices[vertexNum].tu = 0.5f;
			m_vertices[vertexNum].tv = 0.5f;

			vertexNum++;
			m_vertices[vertexNum].x = cos((i + 1)*angleBetween);
			m_vertices[vertexNum].y = sin((i + 1)*angleBetween);
			m_vertices[vertexNum].z = 0.0f;
			m_vertices[vertexNum].nx = 0.0f;
			m_vertices[vertexNum].ny = 0.0f;
			m_vertices[vertexNum].nz = 1.0f;
			m_vertices[vertexNum].tu = (0.5f + (cos((i + 1)*angleBetween) / 2));
			m_vertices[vertexNum].tv = (0.5f + (sin((i + 1)*angleBetween) / 2));

		}

		//Clamp the final vertex to the start point
		m_vertices[vertexNum] = m_vertices[0];

		return true;
	}
	else
	{
		return false;
	}
}

bool Cube::Initialize(int quadsPerFace, int textureRepeat)
{
	//Check if poly count is a square number
	if (quadsPerFace > 0 && sqrt(quadsPerFace) == floor(sqrt(quadsPerFace)))
	{
		//36 is the number of vertices in a quads by the number of faces in a cube
		int vertex_count = 0;
		int vertexNumber = (quadsPerFace * 36);
		m_vertexCount = vertexNumber;
		float subdivisions = sqrt(quadsPerFace);
		m_vertices = new VertexType[m_vertexCount];

		//Create front face
		for (int y = 0; y < subdivisions; y++)
		{
			for (int x = 0; x < subdivisions; x++)
			{
				//One quad
				//Triangle One
				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + (x *(1.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y *(1.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + 1.0f;
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x) *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 1.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + (x *(1.0f / subdivisions)) + (1.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y *(1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + 1.0f;
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 1.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + (x *(1.0f / subdivisions)) + (1.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y *(1.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + 1.0f;
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 1.0f;
				vertex_count++;

				//Triangle Two
				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + (x *(1.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y *(1.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + 1.0f;
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x) *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 1.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + (x *(1.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y *(1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + 1.0f;
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 1.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + (x *(1.0f / subdivisions)) + (1.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y *(1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + 1.0f;
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 1.0f;
				vertex_count++;
			}
		}

		//Create back face
		for (int y = 0; y < subdivisions; y++)
		{
			for (int x = 0; x < subdivisions; x++)
			{
				//One quad
				//Triangle One
				m_vertices[vertex_count].x = (0 + (1.0f / 2)) - (x *(1.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y *(1.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f / 2));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x)) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = -1.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 + (1.0f / 2)) - (x *(1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y *(1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f / 2));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = -1.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 + (1.0f / 2)) - (x *(1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y *(1.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f / 2));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = -1.0f;
				vertex_count++;

				//Triangle Two
				m_vertices[vertex_count].x = (0 + (1.0f / 2)) - (x *(1.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y *(1.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f / 2));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x)) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = -1.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 + (1.0f / 2)) - (x *(1.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y *(1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f / 2));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x)) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = -1.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 + (1.0f / 2)) - (x *(1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y *(1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f / 2));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = -1.0f;
				vertex_count++;
			}
		}

		//Create left face
		for (int y = 0; y < subdivisions; y++)
		{
			for (int z = 0; z < subdivisions; z++)
			{
				//One quad
				//Triangle One
				m_vertices[vertex_count].x = (0 - (1.0f / 2));
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y * (1.0f / subdivisions));
				m_vertices[vertex_count].z = (0 + (1.0f / 2)) - (z * (1.0f / subdivisions));
				m_vertices[vertex_count].tu = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y)  *  textureRepeat;
				m_vertices[vertex_count].nx = -1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f / 2));
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y * (1.0f / subdivisions));
				m_vertices[vertex_count].z = (0 + (1.0f / 2)) - (z * (1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].tu = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y)  *  textureRepeat;
				m_vertices[vertex_count].nx = -1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f / 2));
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y * (1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].z = (0 + (1.0f / 2)) - (z * (1.0f / subdivisions));
				m_vertices[vertex_count].tu = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = -1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				//Triangle Two
				m_vertices[vertex_count].x = (0 - (1.0f / 2));
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y * (1.0f / subdivisions));
				m_vertices[vertex_count].z = (0 + (1.0f / 2)) - (z * (1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].tu = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y)  *  textureRepeat;
				m_vertices[vertex_count].nx = -1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f / 2));
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y * (1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].z = (0 + (1.0f / 2)) - (z * (1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].tu = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = -1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f / 2));
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y * (1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].z = (0 + (1.0f / 2)) - (z * (1.0f / subdivisions));
				m_vertices[vertex_count].tu = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = -1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;
			}
		}

		//Create right face
		for (int y = 0; y < subdivisions; y++)
		{
			for (int z = 0; z < subdivisions; z++)
			{
				//One quad
				//Triangle One
				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + 1.0f;
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y * (1.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions));
				m_vertices[vertex_count].tu = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y)  *  textureRepeat;
				m_vertices[vertex_count].nx = 1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + 1.0f;
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y * (1.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions)) + (1.0f / subdivisions);
				m_vertices[vertex_count].tu = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y)  *  textureRepeat;
				m_vertices[vertex_count].nx = 1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + 1.0f;
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y * (1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions));
				m_vertices[vertex_count].tu = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = 1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				//Triangle Two
				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + 1.0f;
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y * (1.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions)) + (1.0f / subdivisions);
				m_vertices[vertex_count].tu = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y)  *  textureRepeat;
				m_vertices[vertex_count].nx = 1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + 1.0f;
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y * (1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions)) + (1.0f / subdivisions);
				m_vertices[vertex_count].tu = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = 1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + 1.0f;
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - (y * (1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions));
				m_vertices[vertex_count].tu = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = 1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;
			}
		}

		//Create top face
		for (int z = 0; z < subdivisions; z++)
		{
			for (int x = 0; x < subdivisions; x++)
			{
				//One quad
				//Triangle One
				m_vertices[vertex_count].x = (0 + (1.0f / 2)) - (x *(1.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f / 2));
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions));
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x)  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 + (1.0f / 2)) - (x *(1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f / 2));
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 + (1.0f / 2)) - (x *(1.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f / 2));
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions)) + (1.0f / subdivisions);
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x)  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				//Triangle Two
				m_vertices[vertex_count].x = (0 + (1.0f / 2)) - (x *(1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f / 2));
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 + (1.0f / 2)) - (x *(1.0f / subdivisions)) - (1.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f / 2));
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions)) + (1.0f / subdivisions);
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 + (1.0f / 2)) - (x *(1.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f / 2));
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions)) + (1.0f / subdivisions);
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x)  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;
			}
		}

		//Create bottom face
		for (int z = 0; z < subdivisions; z++)
		{
			for (int x = 0; x < subdivisions; x++)
			{
				//One quad
				//Triangle One
				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + (x *(1.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - 1.0f;
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions));
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x)  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = -1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + (x *(1.0f / subdivisions)) + (1.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - 1.0f;
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions))   *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * z)   *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = -1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + (x *(1.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - 1.0f;
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions)) + (1.0f / subdivisions);
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x)   *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * z) + (1 / subdivisions))   *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = -1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				//Triangle Two
				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + (x *(1.0f / subdivisions)) + (1.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - 1.0f;
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions))   *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * z)   *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = -1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + (x *(1.0f / subdivisions)) + (1.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - 1.0f;
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions)) + (1.0f / subdivisions);
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions))   *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * z) + (1 / subdivisions))   *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = -1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f / 2)) + (x *(1.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f / 2)) - 1.0f;
				m_vertices[vertex_count].z = (0 - (1.0f / 2)) + (z * (1.0f / subdivisions)) + (1.0f / subdivisions);
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x)   *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * z) + (1 / subdivisions))   *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = -1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;
			}
		}

		return true;
	}
	else
	{
		return false;
	}
}

bool CubeSphere::Initialize(int quadsPerFace, int textureRepeat)
{
	rising = false;
	lerpS = 0.0f;

	InitCube(quadsPerFace, textureRepeat);

	for (int i = 0; i < m_vertexCount; i++)
	{
		float x = m_vertices[i].x;
		float y = m_vertices[i].y;
		float z = m_vertices[i].z;

		m_vertices[i].iniX = x;
		m_vertices[i].iniY = y;
		m_vertices[i].iniZ = z;

		m_vertices[i].iniNx = m_vertices[i].nx;
		m_vertices[i].iniNy = m_vertices[i].ny;
		m_vertices[i].iniNz = m_vertices[i].nz;

		m_vertices[i].lerpX = x * sqrtf(1 - ((y*y) / 2) - ((z*z) / 2) + (((y*y)*(z*z)) / 3));
		m_vertices[i].lerpY = y * sqrtf(1 - ((z*z) / 2) - ((x*x) / 2) + (((x*x)*(z*z)) / 3));
		m_vertices[i].lerpZ = z * sqrtf(1 - ((x*x) / 2) - ((y*y) / 2) + (((x*x)*(y*y)) / 3));

		m_vertices[i].nx = m_vertices[i].lerpX;
		m_vertices[i].ny = m_vertices[i].lerpY;
		m_vertices[i].nz = m_vertices[i].lerpZ;
	}

	return true;

	return true;
}

bool CubeSphere::InitCube(int quadsPerFace, int textureRepeat)
{
	//Check if poly count is a square number
	if (quadsPerFace > 0 && sqrt(quadsPerFace) == floor(sqrt(quadsPerFace)))
	{
		//36 is the number of vertices in a quads by the number of faces in a cube
		int vertex_count = 0;
		int vertexNumber = (quadsPerFace * 36);
		m_vertexCount = vertexNumber;
		float subdivisions = sqrt(quadsPerFace);
		m_vertices = new VertexType[m_vertexCount];

		//Create front face
		for (int y = 0; y < subdivisions; y++)
		{
			for (int x = 0; x < subdivisions; x++)
			{
				//One quad
				//Triangle One
				m_vertices[vertex_count].x = (0 - (1.0f)) + (x *(2.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y *(2.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f)) + 2.0f;
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x) *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 1.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f)) + (x *(2.0f / subdivisions)) + (2.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y *(2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f)) + 2.0f;
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 1.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f)) + (x *(2.0f / subdivisions)) + (2.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y *(2.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f)) + 2.0f;
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 1.0f;
				vertex_count++;

				//Triangle Two
				m_vertices[vertex_count].x = (0 - (1.0f)) + (x *(2.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y *(2.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f)) + 2.0f;
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x) *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 1.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f)) + (x *(2.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y *(2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f)) + 2.0f;
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 1.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f)) + (x *(2.0f / subdivisions)) + (2.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y *(2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f)) + 2.0f;
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 1.0f;
				vertex_count++;
			}
		}

		//Create back face
		for (int y = 0; y < subdivisions; y++)
		{
			for (int x = 0; x < subdivisions; x++)
			{
				//One quad
				//Triangle One
				m_vertices[vertex_count].x = (0 + (1.0f)) - (x *(2.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y *(2.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x)) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = -1.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 + (1.0f)) - (x *(2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y *(2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = -1.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 + (1.0f)) - (x *(2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y *(2.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = -1.0f;
				vertex_count++;

				//Triangle Two
				m_vertices[vertex_count].x = (0 + (1.0f)) - (x *(2.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y *(2.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x)) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = -1.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 + (1.0f)) - (x *(2.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y *(2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x)) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = -1.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 + (1.0f)) - (x *(2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y *(2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions)) *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = -1.0f;
				vertex_count++;
			}
		}

		//Create left face
		for (int y = 0; y < subdivisions; y++)
		{
			for (int z = 0; z < subdivisions; z++)
			{
				//One quad
				//Triangle One
				m_vertices[vertex_count].x = (0 - (1.0f));
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y * (2.0f / subdivisions));
				m_vertices[vertex_count].z = (0 + (1.0f)) - (z * (2.0f / subdivisions));
				m_vertices[vertex_count].tu = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y)  *  textureRepeat;
				m_vertices[vertex_count].nx = -1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f));
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y * (2.0f / subdivisions));
				m_vertices[vertex_count].z = (0 + (1.0f)) - (z * (2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].tu = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y)  *  textureRepeat;
				m_vertices[vertex_count].nx = -1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f));
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y * (2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].z = (0 + (1.0f)) - (z * (2.0f / subdivisions));
				m_vertices[vertex_count].tu = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = -1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				//Triangle Two
				m_vertices[vertex_count].x = (0 - (1.0f));
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y * (2.0f / subdivisions));
				m_vertices[vertex_count].z = (0 + (1.0f)) - (z * (2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].tu = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y)  *  textureRepeat;
				m_vertices[vertex_count].nx = -1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f));
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y * (2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].z = (0 + (1.0f)) - (z * (2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].tu = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = -1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f));
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y * (2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].z = (0 + (1.0f)) - (z * (2.0f / subdivisions));
				m_vertices[vertex_count].tu = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = -1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;
			}
		}

		//Create right face
		for (int y = 0; y < subdivisions; y++)
		{
			for (int z = 0; z < subdivisions; z++)
			{
				//One quad
				//Triangle One
				m_vertices[vertex_count].x = (0 - (1.0f)) + 2.0f;
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y * (2.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions));
				m_vertices[vertex_count].tu = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y)  *  textureRepeat;
				m_vertices[vertex_count].nx = 1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f)) + 2.0f;
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y * (2.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions)) + (2.0f / subdivisions);
				m_vertices[vertex_count].tu = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y)  *  textureRepeat;
				m_vertices[vertex_count].nx = 1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f)) + 2.0f;
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y * (2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions));
				m_vertices[vertex_count].tu = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = 1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				//Triangle Two
				m_vertices[vertex_count].x = (0 - (1.0f)) + 2.0f;
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y * (2.0f / subdivisions));
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions)) + (2.0f / subdivisions);
				m_vertices[vertex_count].tu = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * y)  *  textureRepeat;
				m_vertices[vertex_count].nx = 1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f)) + 2.0f;
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y * (2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions)) + (2.0f / subdivisions);
				m_vertices[vertex_count].tu = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = 1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f)) + 2.0f;
				m_vertices[vertex_count].y = (0 + (1.0f)) - (y * (2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions));
				m_vertices[vertex_count].tu = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * y) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = 1.0f;
				m_vertices[vertex_count].ny = 0.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;
			}
		}

		//Create top face
		for (int z = 0; z < subdivisions; z++)
		{
			for (int x = 0; x < subdivisions; x++)
			{
				//One quad
				//Triangle One
				m_vertices[vertex_count].x = (0 + (1.0f)) - (x *(2.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f));
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions));
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x)  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 + (1.0f)) - (x *(2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f));
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 + (1.0f)) - (x *(2.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f));
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions)) + (2.0f / subdivisions);
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x)  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				//Triangle Two
				m_vertices[vertex_count].x = (0 + (1.0f)) - (x *(2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f));
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 + (1.0f)) - (x *(2.0f / subdivisions)) - (2.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f));
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions)) + (2.0f / subdivisions);
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 + (1.0f)) - (x *(2.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f));
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions)) + (2.0f / subdivisions);
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x)  *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * z) + (1 / subdivisions))  *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = 1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;
			}
		}

		//Create bottom face
		for (int z = 0; z < subdivisions; z++)
		{
			for (int x = 0; x < subdivisions; x++)
			{
				//One quad
				//Triangle One
				m_vertices[vertex_count].x = (0 - (1.0f)) + (x *(2.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f)) - 2.0f;
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions));
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x)  *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * z)  *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = -1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f)) + (x *(2.0f / subdivisions)) + (2.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f)) - 2.0f;
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions))   *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * z)   *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = -1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f)) + (x *(2.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f)) - 2.0f;
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions)) + (2.0f / subdivisions);
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x)   *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * z) + (1 / subdivisions))   *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = -1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				//Triangle Two
				m_vertices[vertex_count].x = (0 - (1.0f)) + (x *(2.0f / subdivisions)) + (2.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f)) - 2.0f;
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions));
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions))   *  textureRepeat;
				m_vertices[vertex_count].tv = ((1 / subdivisions) * z)   *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = -1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f)) + (x *(2.0f / subdivisions)) + (2.0f / subdivisions);
				m_vertices[vertex_count].y = (0 + (1.0f)) - 2.0f;
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions)) + (2.0f / subdivisions);
				m_vertices[vertex_count].tu = (((1 / subdivisions) * x) + (1 / subdivisions))   *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * z) + (1 / subdivisions))   *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = -1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;

				m_vertices[vertex_count].x = (0 - (1.0f)) + (x *(2.0f / subdivisions));
				m_vertices[vertex_count].y = (0 + (1.0f)) - 2.0f;
				m_vertices[vertex_count].z = (0 - (1.0f)) + (z * (2.0f / subdivisions)) + (2.0f / subdivisions);
				m_vertices[vertex_count].tu = ((1 / subdivisions) * x)   *  textureRepeat;
				m_vertices[vertex_count].tv = (((1 / subdivisions) * z) + (1 / subdivisions))   *  textureRepeat;
				m_vertices[vertex_count].nx = 0.0f;
				m_vertices[vertex_count].ny = -1.0f;
				m_vertices[vertex_count].nz = 0.0f;
				vertex_count++;
			}
		}

		return true;
	}
	else
	{
		return false;
	}
}

void CubeSphere::LerpCubeSphere(float dt)
{
	if (rising)
	{
		lerpS += dt;
		if (lerpS >= 1.0f)
		{
			lerpS = 1.0f;
			rising = false;
		}
	}
	else
	{
		lerpS -= dt;
		if (lerpS <= 0.0f)
		{
			lerpS = 0.0f;
			rising = true;
		}
	}

	for (int i = 0; i < m_vertexCount; i++)
	{
		m_vertices[i].x = lerp(m_vertices[i].iniX, m_vertices[i].lerpX, lerpS);
		m_vertices[i].y = lerp(m_vertices[i].iniY, m_vertices[i].lerpY, lerpS);
		m_vertices[i].z = lerp(m_vertices[i].iniZ, m_vertices[i].lerpZ, lerpS);

		m_vertices[i].nx = lerp(m_vertices[i].iniNx, m_vertices[i].lerpX, lerpS);
		m_vertices[i].ny = lerp(m_vertices[i].iniNy, m_vertices[i].lerpY, lerpS);
		m_vertices[i].nz = lerp(m_vertices[i].iniNz, m_vertices[i].lerpZ, lerpS);
	}
}

bool GeodesicSphere::Initialize(int resolution)
{
	current_index_count = 0;
	current_vertex_count = 0;

	m_indexCount = 0;

	float X = .525731112119133606;
	float Z = .850650808352039932;

	isIndexed = true;

	static float vData[12][3] =
	{
		{ -X, 0.0, Z }, { X, 0.0, Z }, { -X, 0.0, -Z }, { X, 0.0, -Z },
		{ 0.0, Z, X }, { 0.0, Z, -X }, { 0.0, -Z, X }, { 0.0, -Z, -X },
		{ Z, X, 0.0 }, { -Z, X, 0.0 }, { Z, -X, 0.0 }, { -Z, -X, 0.0 }
	};

	static uint16_t tIndices[20][3] =
	{
		{ 0, 4, 1 }, { 0, 9, 4 }, { 9, 5, 4 }, { 4, 5, 8 }, { 4, 8, 1 },
		{ 8, 10, 1 }, { 8, 3, 10 }, { 5, 3, 8 }, { 5, 2, 3 }, { 2, 7, 3 },
		{ 7, 10, 3 }, { 7, 6, 10 }, { 7, 11, 6 }, { 11, 0, 6 }, { 0, 1, 6 },
		{ 6, 1, 10 }, { 9, 0, 11 }, { 9, 11, 2 }, { 9, 2, 5 }, { 7, 2, 11 }
	};

	m_vertexCount = GeodesicVertexCount(resolution);
	m_indexCount = (pow(4, resolution) * 3) * 20;

	m_vertices = new VertexType[m_vertexCount];
	m_indices = new long[m_indexCount];

	for (int i = 0; i < 20; i++)
	{
		subdivide_and_index(vData[tIndices[i][0]], vData[tIndices[i][1]], vData[tIndices[i][2]], resolution);
	}

	for (int i = 0; i < m_vertexCount; i++)
	{
		m_vertices[i].iniX = m_vertices[i].x;
		m_vertices[i].iniY = m_vertices[i].y;
		m_vertices[i].iniZ = m_vertices[i].z;

		m_vertices[i].lerpX = m_vertices[i].x / 1.5;
		m_vertices[i].lerpY = m_vertices[i].y / 1.5;
		m_vertices[i].lerpZ = m_vertices[i].z / 1.5;

		m_vertices[i].lerpS = static_cast<float> (rand()) / static_cast<float>(RAND_MAX);

		if (int(m_vertices[i].lerpS * 10) % 2 == 0)
		{
			m_vertices[i].lerpD = true;
		}
		else
		{
			m_vertices[i].lerpD = false;
		}
	}

	return true;
}

void GeodesicSphere::Render()
{
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, GetTexture());
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	float x, y, z;

	glPushMatrix();
	GetScaling(x, y, z);
	glScalef(x, y, z);
	GetTranslation(x, y, z);
	glTranslatef(x, y, z);
	GetRotation(x, y, z);
	glRotatef(z, 0, 0, 1);
	glRotatef(y, 0, 1, 0);
	glRotatef(x, 1, 0, 0);

	glBegin(GL_TRIANGLES);

	for (int i = 0; i < GetIndexCount(); ++i)
	{
		long index = GetIndex(i);

		glNormal3f(m_vertices[index].nx, m_vertices[index].ny, m_vertices[index].nz);
		glMultiTexCoord2f(GL_TEXTURE0, m_vertices[index].tu, m_vertices[index].tv);
		glMultiTexCoord2f(GL_TEXTURE1, m_vertices[index].tu, m_vertices[index].tv);
		glVertex3f(m_vertices[index].x, m_vertices[index].y, m_vertices[index].z);
	}

	glEnd();
	glPopMatrix();
}

int GeodesicSphere::GeodesicVertexCount(int subdivisions)
{
	int num_unique_vertices = 0;
	int total_num_vertices = 0;
	int absolute_total_vertices = 0;
	int num_edge_vertices = 2;
	int vertex_additive = 0;

	for (int i = 1; i <= subdivisions; i++)
	{
		num_edge_vertices += pow(2, (i - 1));
	}

	for (int i = 1; i <= num_edge_vertices; i++)
	{
		total_num_vertices += i;
	}

	num_unique_vertices = total_num_vertices - num_edge_vertices - (num_edge_vertices - 1) - (num_edge_vertices - 2);

	absolute_total_vertices = (30 * num_edge_vertices) + (20 * num_unique_vertices);

	return absolute_total_vertices;
}

void GeodesicSphere::subdivide_and_index(float v1[3], float v2[3], float v3[3], long depth)
{
	float v12[3], v23[3], v31[3];

	bool v1found = false,
		v2found = false,
		v3found = false;

	if (current_index_count > m_indexCount || current_vertex_count > m_vertexCount)
	{
		MessageBox(NULL, "Something went wrong while generating the sphere.", "Error", MB_OK);
		return;
	}

	if (depth == 0)
	{
		for (int i = 0; i < current_vertex_count; i++)
		{
			if (!v1found)
			{
				if (m_vertices[i].x == v1[0] && m_vertices[i].y == v1[1] && m_vertices[i].z == v1[2])
				{
					v1found = true;
					m_indices[current_index_count] = i;
				}
			}

			if (!v2found)
			{
				if (m_vertices[i].x == v2[0] && m_vertices[i].y == v2[1] && m_vertices[i].z == v2[2])
				{
					v2found = true;
					m_indices[current_index_count + 1] = i;
				}
			}

			if (!v3found)
			{
				if (m_vertices[i].x == v3[0] && m_vertices[i].y == v3[1] && m_vertices[i].z == v3[2])
				{
					v3found = true;
					m_indices[current_index_count + 2] = i;
				}
			}
		}

		if (!v1found)
		{
			m_vertices[current_vertex_count].x = v1[0];
			m_vertices[current_vertex_count].y = v1[1];
			m_vertices[current_vertex_count].z = v1[2];

			m_vertices[current_vertex_count].iniNx = v1[0];
			m_vertices[current_vertex_count].iniNy = v1[1];
			m_vertices[current_vertex_count].iniNz = v1[2];

			m_vertices[current_vertex_count].tu = atan(v1[0] / v1[2]);
			m_vertices[current_vertex_count].tv = acos(v1[1]);

			m_indices[current_index_count] = current_vertex_count;

			current_vertex_count++;
		}

		current_index_count++;

		if (!v2found)
		{
			m_vertices[current_vertex_count].x = v2[0];
			m_vertices[current_vertex_count].y = v2[1];
			m_vertices[current_vertex_count].z = v2[2];

			m_vertices[current_vertex_count].iniNx = v2[0];
			m_vertices[current_vertex_count].iniNy = v2[1];
			m_vertices[current_vertex_count].iniNz = v2[2];

			m_vertices[current_vertex_count].tu = atan(v2[0] / v2[2]);
			m_vertices[current_vertex_count].tv = acos(v2[1]);
			
			m_indices[current_index_count] = current_vertex_count;

			current_vertex_count++;
		}

		current_index_count++;

		if (!v3found)
		{
			m_vertices[current_vertex_count].x = v3[0];
			m_vertices[current_vertex_count].y = v3[1];
			m_vertices[current_vertex_count].z = v3[2];

			m_vertices[current_vertex_count].iniNx = v3[0];
			m_vertices[current_vertex_count].iniNy = v3[1];
			m_vertices[current_vertex_count].iniNz = v3[2];

			m_vertices[current_vertex_count].tu = atan(v3[0] / v3[2]);
			m_vertices[current_vertex_count].tv = acos(v3[1]);

			m_indices[current_index_count] = current_vertex_count;

			current_vertex_count++;
		}

		current_index_count++;

		return;
	}

	for (int i = 0; i < 3; i++)
	{
		v12[i] = v1[i] + v2[i];
		v23[i] = v2[i] + v3[i];
		v31[i] = v3[i] + v1[i];
	}

	normalize(v12);
	normalize(v23);
	normalize(v31);

	subdivide_and_index(v1, v12, v31, depth - 1);
	subdivide_and_index(v2, v23, v12, depth - 1);
	subdivide_and_index(v3, v31, v23, depth - 1);
	subdivide_and_index(v12, v23, v31, depth - 1);
}

void GeodesicSphere::GeodesicSphereNoise(float dt)
{
	for (int i = 0; i < m_vertexCount; i++)
	{
		if (m_vertices[i].lerpD)
		{
			m_vertices[i].lerpS += dt;
			if (m_vertices[i].lerpS >= 1.0f)
			{
				m_vertices[i].lerpS = 1.0f;
				m_vertices[i].lerpD = false;
			}
		}
		else
		{
			m_vertices[i].lerpS -= dt;
			if (m_vertices[i].lerpS <= 0.0f)
			{
				m_vertices[i].lerpS = 0.0f;
				m_vertices[i].lerpD = true;
			}
		}

		m_vertices[i].x = lerp(m_vertices[i].iniX, m_vertices[i].lerpX, m_vertices[i].lerpS);
		m_vertices[i].nx = lerp(m_vertices[i].iniNx, m_vertices[i].lerpX, m_vertices[i].lerpS);

		m_vertices[i].y = lerp(m_vertices[i].iniY, m_vertices[i].lerpY, m_vertices[i].lerpS);
		m_vertices[i].ny = lerp(m_vertices[i].iniNy, m_vertices[i].lerpY, m_vertices[i].lerpS);

		m_vertices[i].z = lerp(m_vertices[i].iniZ, m_vertices[i].lerpZ, m_vertices[i].lerpS);
		m_vertices[i].nz = lerp(m_vertices[i].iniNz, m_vertices[i].lerpZ, m_vertices[i].lerpS);
	}
}