#include "Scene3D.h"

bool Scene3D::CreatePixelFormat(HDC hdc) 
{ 
    PIXELFORMATDESCRIPTOR pfd = {0}; 
    int pixelformat; 
 
    pfd.nSize = sizeof(PIXELFORMATDESCRIPTOR);	// Set the size of the structure
    pfd.nVersion = 1;							// Always set this to 1
	// Pass in the appropriate OpenGL flags
    pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER; 
    pfd.dwLayerMask = PFD_MAIN_PLANE;			// standard mask (this is ignored anyway)
    pfd.iPixelType = PFD_TYPE_RGBA;				// RGB and Alpha pixel type
    pfd.cColorBits = COLOUR_DEPTH;				// Here we use our #define for the color bits
    pfd.cDepthBits = COLOUR_DEPTH;				// Ignored for RBA
    pfd.cAccumBits = 0;							// nothing for accumulation
    pfd.cStencilBits = 0;						// nothing for stencil
 
	//Gets a best match on the pixel format as passed in from device
    if ( (pixelformat = ChoosePixelFormat(hdc, &pfd)) == false ) 
    { 
        MessageBox(NULL, "ChoosePixelFormat failed", "Error", MB_OK); 
        return false; 
    } 
 
	//sets the pixel format if its ok. 
    if (SetPixelFormat(hdc, pixelformat, &pfd) == false) 
    { 
        MessageBox(NULL, "SetPixelFormat failed", "Error", MB_OK); 
        return false; 
    } 
 
    return true;
}

void Scene3D::ResizeGLWindow(int width, int height)// Initialize The GL Window
{
	if (height==0)// Prevent A Divide By Zero error
	{
		height=1;// Make the Height Equal One
	}

	s_width = width;
	s_height = height;

	glViewport(0,0,width,height);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	//calculate aspect ratio
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height, 0.01f , 300.0f);

	glMatrixMode(GL_MODELVIEW);// Select The Modelview Matrix
	glLoadIdentity();// Reset The Modelview Matrix
}

void Scene3D::UpdateProjection(bool toggle)
{
	static bool usePerspective = true;

	if (toggle)
		usePerspective = !usePerspective;

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	if (usePerspective)
	{
		gluPerspective(45.0f, (GLfloat)s_width / (GLfloat)s_height, 0.01f, 300.0f);
	}
	else
	{
		gluOrtho2D(-(s_width/2), (s_width/2), -(s_height/2), (s_height/2));
	}

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glViewport(0, 0, s_width, s_height);
}

void Scene3D::InitializeOpenGL(int width, int height) 
{  
    hdc = GetDC(*hwnd);//  sets  global HDC

    if (!CreatePixelFormat(hdc))//  sets  pixel format
        PostQuitMessage (0);

    hrc = wglCreateContext(hdc);	//  creates  rendering context from  hdc
    wglMakeCurrent(hdc, hrc);		//	Use this HRC

	ResizeGLWindow(width, height);	// Setup the Screen
}

void Scene3D::Init(HWND* wnd, Input* in)
{
	DirectionalLightData directionalData;
	PointLightData pointLight;
	SpotLightData spotLight;
	SpotLightData flashLight;
	hwnd = wnd;
	input = in;
	rotation = 0;
	water_translation = 0.0f;
	cumulativeDtime = 0;
	frameEnabled = true;

	light_names[0] = GL_LIGHT0;
	light_names[1] = GL_LIGHT1;
	light_names[2] = GL_LIGHT2;
	light_names[3] = GL_LIGHT3;
	light_names[4] = GL_LIGHT4;
	light_names[5] = GL_LIGHT5;
	light_names[6] = GL_LIGHT6;
	light_names[7] = GL_LIGHT7;

	ambientLight.r = 0.1f;
	ambientLight.g = 0.1f;
	ambientLight.b = 0.1f;
	ambientLight.a = 1.0f;

	directionalData.r = 0.01f;
	directionalData.g = 0.01f;
	directionalData.b = 0.01f;
	directionalData.a = 1.0f;

	directionalData.dirX = 1.0f;
	directionalData.dirY = 1.0f;
	directionalData.dirZ = 0.0f;

	directionalLights.push_back(directionalData);

	pointLight.r = 1.0f;
	pointLight.g = 1.0f;
	pointLight.b = 0.45f;

	pointLight.posX = -4.0f;
	pointLight.posY = 10.0f;
	pointLight.posZ = 0.0f;

	pointLight.const_attenuation = 20.0f;
	pointLight.linear_attenuation = 20.0f;
	pointLight.quadratic_attenuation = 20.0f;

	pointLights.push_back(pointLight);

	spotLight.r = 1.0f;
	spotLight.g = 1.0f;
	spotLight.b = 1.0f;
	spotLight.a = 1.0f;

	spotLight.dirX = 0.0f;
	spotLight.dirY = -1.0f;
	spotLight.dirZ = 0.0f;

	spotLight.posX = 20.0f;
	spotLight.posY = 40.0f;
	spotLight.posZ = 0.0f;

	spotLight.linear_attenuation = 1.0f;
	spotLight.spot_exponent = 0.0f;
	spotLight.spot_cutoff = 15.0f;
	spotLight.quadratic_attenuation = 1.0f;

	spotLights.push_back(spotLight);

	flashLight.r = 1.0f;
	flashLight.g = 1.0f;
	flashLight.b = 1.0f;
	flashLight.a = 1.0f;

	flashLight.dirX = 0.0f;
	flashLight.dirY = -1.0f;
	flashLight.dirZ = 0.0f;

	flashLight.posX = 20.0f;
	flashLight.posY = 40.0f;
	flashLight.posZ = 0.0f;

	flashLight.linear_attenuation = 20.0f;
	flashLight.spot_exponent = 20.0f;
	flashLight.spot_cutoff = 10.0f;
	flashLight.quadratic_attenuation = 20.0f;

	spotLights.push_back(flashLight);

	GetClientRect(*hwnd, &screenRect);	//get rect into our handy global rect
	InitializeOpenGL(screenRect.right, screenRect.bottom); // initialise openGL

	//OpenGL settings
	glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
	//glClearColor(0.39f, 0.58f, 93.0f, 1.0f);				// Black Background
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glClearDepth(1.0f);									// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);								// The Type Of Depth Testing To Do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);	// Use faster perspective calculations to save time
	glEnable(GL_LIGHTING);
	//glEnable(GL_COLOR_MATERIAL);
	glewInit();

	m_camera.Initialize(1.0f, 3.0f, 0.0f);
	m_stationaryCamera.Initialize(1.0f, 3.0f, 0.0f);
	m_stationaryCamera.LockMove(1, 1, 1, 3.0f);

	m_topDownCamera.Initialize(m_camera.GetPositionPointer(), 0.0f, 50.0f, 0.0f);

	InitializeReflectionBuffer();
	InitializeRefractionBuffer();

	m_Font = new Font;

	m_Font->Initialize("data/font.png", "data/fontdata.txt");

	m_UIText = new FontModel;

	m_UIText->Initialize(m_Font, "Hello World!", -(s_width / 2) + 10.0f, (s_height / 2) - 230.0f, 1.0f, 1.0f, 1.0f);

	m_Minimap = new CameraViewModel;
	
	m_Minimap->Initialize(&m_topDownCamera, 256, 256, -(s_width/2) + 110.0f, (s_height/2) - 110.0f);

	m_Minimap->SetScaling(200, 200, 1);

	m_viewWindow = new CameraViewModel;

	m_viewWindow->Initialize(&m_camera, 256, 256, 0.0f, 0.0f);

	m_viewWindow->SetScaling(256, 256, 1);

	m_shadowImposter = SOIL_load_OGL_texture("data/imposter.png", SOIL_LOAD_AUTO, SOIL_CREATE_NEW_ID, SOIL_FLAG_MIPMAPS | SOIL_FLAG_NTSC_SAFE_RGB | SOIL_FLAG_COMPRESS_TO_DXT);

	//m_Minimap->LoadTexture("crate.png");

	m_Cube = new Cube;

	static_cast<Cube*>(m_Cube)->Initialize(16, 2);

	m_Cube->LoadTexture("data/crate.png");

	m_Cube->SetTranslation(0.0f, 1.0f, 0.0f);

	m_Skybox = new SkyboxModel;

	m_Skybox->Initialize();

	m_Skybox->SetScaling(20.0f, 20.0f, 20.0f);

	m_sphere = new GeodesicSphere;

	m_sphere->Initialize(4);

	m_sphere->SetScaling(20.0f, 20.0f, 20.0f);
	m_sphere->SetTranslation(-6.0f, 2.0f, 0.0f);
	
	m_sphere->LoadTexture("data/tissue.png");

	m_cubeSphere = new CubeSphere;

	m_cubeSphere->Initialize(16, 2);

	m_cubeSphere->SetScaling(3.0f, 3.0f, 3.0f);
	m_cubeSphere->SetTranslation(20.0f, 15.0f, 0.0f);

	m_cubeSphere->LoadTexture("data/tissue.png");

	m_shadowPlane = new Plane;
	m_shadowPlane->Initialize(144, 1);
	m_shadowPlane->SetScaling(20.0f, 2.0f, 20.0f);
	m_shadowPlane->SetTranslation(20.0f, 10.0f, 0.0f);

	numObjects = 2;

	m_Objects = new Model*[numObjects];

	m_Objects[0] = new TerrainModel;

	m_terrain = m_Objects[0];

	the_clock::time_point begin = the_clock::now();

	static_cast<TerrainModel*>(m_Objects[0])->Initialize("data/flatland.r16");

	the_clock::time_point finish = the_clock::now();

	auto full_time = duration_cast<milliseconds>(finish - begin).count();
	
	std::ostringstream os_;
	os_ << "Building the terrain took " << full_time << " ms." << endl;
	OutputDebugString(os_.str().c_str());
	
	m_Objects[0]->SetTranslation(0.0f, 0.0f, 0.0f);

	m_Objects[0]->LoadTexture("data/rock.png");

	m_Objects[1] = new Plane;

	static_cast<Plane*>(m_Objects[1])->InitializeIndexed(16, 1);

	m_Objects[1]->SetTranslation(-2.0f, -4.0f, 2.0f);

	m_Objects[1]->SetScaling(10.0f, 10.0f, 10.0f);

	m_Objects[1]->LoadTexture("data/crate.png");

	m_Objects[1]->LoadBumpMap("data/bumpmap.bmp");

	m_water = m_Objects[1];

	m_ObjModel = new ObjModel;

	m_ObjModel->Initialize("data/laubbaum.obj");

	m_ObjModel->LoadTexture("data/texture_laubbaum.png");

	m_ObjModel->SetTranslation(0.0f, 2.0f, 8.0f);

	m_ObjModel->SetShadowImposter(m_shadowImposter);

	m_ObjModel->CreateShadowImposter(m_Objects[0]);

	m_flashlight = new ObjModel;

	m_flashlight->Initialize("data/torch.obj");

	m_flashlight->LoadTexture("data/flashlight.png");

	m_flashlight->SetScaling(0.001f, 0.001f, 0.001f);

	m_hModel = new HModel;

	m_hModel->Initialize(m_Font, "Hierarchy");

	float light_pos[4] = { -4.0f, 5.0f, 0.0f, 1.0f };

	float objX, objZ, objHeight;

	m_ObjModel->GetTranslation(objX, objHeight, objZ);

	bool foundHeight = static_cast<TerrainModel*>(m_terrain)->GetHeightAtPosition(objX, objZ, objHeight);
	if (foundHeight)
	{
		m_ObjModel->SetTranslation(objX, objHeight, objZ);
	}

	//m_ObjModel->GenerateShadow(light_pos, m_Objects[0]);

	//Enable texturing and set the texturing parameters
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	GLfloat density = 0.03f;

	GLfloat fogColour[4] = { 0.5, 0.5, 0.5, 1.0 };
	
	glEnable(GL_FOG);
	glFogi(GL_FOG_MODE, GL_EXP2);
	glFogfv(GL_FOG_COLOR, fogColour);
	glFogf(GL_FOG_DENSITY, density);
	glHint(GL_FOG_HINT, GL_DONT_CARE);

}

bool Scene3D::Update(float dt)
{
	bool foundHeight;
	float height, fps;
	char tempString[16];
	char fpsString[16];
	Vector3 prevPosition;

	spotAngle += (dt*(20.0f* (3.14f / 180.0f)));

	Vector3 target_position(20.0f, 10.0f, 0.0f);

	spotLights[0].posX = 20.0f + (40.0f * sin(spotAngle));
	spotLights[0].posZ = 0.0f + (40.0f * cos(spotAngle));

	Vector3 light_posit = Vector3(spotLights[0].posX, spotLights[0].posY, spotLights[0].posZ);

	Vector3 light_dir = light_posit - target_position;

	spotLights[0].dirX = -light_dir.x;
	spotLights[0].dirY = -light_dir.y;
	spotLights[0].dirZ = -light_dir.z;

	fps = 1.0f / dt;

	if (fps > 9999)
	{
		fps = 9999;
	}

	_itoa_s(fps, tempString, 10);

	strcpy_s(fpsString, "Fps: ");
	strcat_s(fpsString, tempString);

	m_UIText->UpdateSentence(fpsString);

	// Do important update here
	// Handle user input, update variables etc
	cumulativeDtime += dt;

	texture_translation -= (0.001f * (dt * 1000));

	water_translation += dt;

	if (frameEnabled)
	{
		for (int i = 0; i < numObjects; i++)
		{
			m_Objects[i]->CalculateTransformMatrix();
		}
	}

	m_Cube->CalculateTransformMatrix();
	m_shadowPlane->CalculateTransformMatrix();

	if (input->isKeyDown(VK_SPACE))
	{
		input->SetKeyUp(VK_SPACE);
		//frameEnabled = !frameEnabled;
		followTerrain = !followTerrain;
		cumulativeDtime -= 1.0f;
	}

	if (input->isKeyDown(VK_TAB))
	{
		input->SetKeyUp(VK_TAB);
		wireframeMode = !wireframeMode;
		cumulativeDtime -= 1.0f;
	}

	if (input->isKeyDown(VK_RETURN))
	{
		input->SetKeyUp(VK_RETURN);
		m_stationaryCam = !m_stationaryCam;
		cumulativeDtime -= 1.0f;
	}

	if (input->isKeyDown(VK_ESCAPE))
	{
		return true;
	}

	prevPosition = m_camera.GetPosition();

	m_camera.Frame(input, dt);
	m_stationaryCamera.Frame(input, dt);
	m_topDownCamera.Frame(input, dt);

	m_Skybox->Frame(dt);

	m_sphere->GeodesicSphereNoise(dt);
	m_cubeSphere->LerpCubeSphere(dt);

	Vector3 position = m_camera.GetPosition();
	Vector3 normal;
	if (followTerrain)
	{
		foundHeight = static_cast<TerrainModel*>(m_terrain)->GetHeightAtPosition(position.x, position.z, height);
		if (foundHeight)
		{
			m_camera.SetPosition(Vector3(position.x, height + 2.0f, position.z));
		}

		foundHeight = reinterpret_cast<TerrainModel*>(m_terrain)->GetNormalAtPosition(position.x, position.z, normal);
		if (foundHeight && normal.y < 0.65)
		{
			m_camera.SetPosition(prevPosition);
		}
	}

	m_Minimap->Frame();
	m_viewWindow->Frame();

	position = m_camera.GetPosition();

	Vector3 forward = m_camera.GetForward();
	Vector3 up = m_camera.GetUp();
	Vector3 right = m_camera.GetRight();

	spotLights[1].posX = position.x + (right.x * 0.8f);
	spotLights[1].posY = position.y + (right.y * 0.8f);
	spotLights[1].posZ = position.z + (right.z * 0.8f);

	spotLights[1].dirX = forward.x;
	spotLights[1].dirY = forward.y;
	spotLights[1].dirZ = forward.z;

	m_flashlight->SetTranslation(position.x - (forward.x * 0.05f) - (up.x * 0.03f), position.y - (forward.y * 0.05f) - (up.y * 0.03f), position.z - (forward.z * 0.05f) - (up.z * 0.03f));
	Vector3 rotation = m_camera.GetRotation();
	m_flashlight->SetRotation(rotation.x, -rotation.y, rotation.z);
	m_flashlight->CalculateTransformMatrix();

	m_hModel->Frame(input, dt);

	Render(m_Minimap->GetCamera(), m_Minimap->GetDrawBuffer(), false, false, m_Minimap->GetViewWidth(), m_Minimap->GetViewHeight());

	if (wireframeMode)
		Render(m_viewWindow->GetCamera(), m_viewWindow->GetDrawBuffer(), true, false, m_viewWindow->GetViewWidth(), m_viewWindow->GetViewHeight());

	UpdateProjection(false);

	// Render scene once all updating has been complete
	if (m_stationaryCam)
	{
		Render(&m_stationaryCamera, 0, false, true, 0, 0);
	}
	else
	{
		Render(&m_camera, 0, false, true, 0, 0);

	}

	return false;
}

void Scene3D::Render(CameraClass* camera, GLuint drawBuffer, bool wireframe, bool renderUI, int width, int height)
{
	glBindFramebuffer(GL_FRAMEBUFFER, drawBuffer);
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);// Clear The Screen And The Depth Buffer
	
	glLoadIdentity();

	if (width && height)
	{
		glViewport(0, 0, width, height);

		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();

		//calculate aspect ratio
		gluPerspective(45.0f * ((float)height/(float)s_height), (GLfloat)width / (GLfloat)height, 0.01f, 300.0f);

		glMatrixMode(GL_MODELVIEW);// Select The Modelview Matrix
		glLoadIdentity();// Reset The Modelview Matrix
	}

	glLoadIdentity();// load Identity Matrix

	Vector3 position = camera->GetPosition();
	Vector3 lookAt = camera->GetLookAt();
	Vector3 up = camera->GetUp();
	
	if (wireframe)
	{
		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	}
	else
	{
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	}

	gluLookAt(position.x, position.y, position.z, lookAt.x, lookAt.y, lookAt.z, up.x, up.y, up.z);

	m_Skybox->SetTranslation(position.x, position.y, position.z);

	m_Skybox->CalculateTransformMatrix();

	m_Skybox->Render();

	SetLights();

	m_Cube->Render();
	m_shadowPlane->Render();
	m_sphere->Render();

	float posit[] = { spotLights[0].posX, spotLights[0].posY, spotLights[0].posZ, 1.0f };

	m_cubeSphere->RenderPlanarShadow(posit);
	m_cubeSphere->Render();

	for (int j = 0; j < numObjects; j++)
	{
		m_Objects[j]->Render();
	}

	//m_ObjModel->RenderShadow();

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	m_ObjModel->RenderShadow();
	m_ObjModel->Render();
	glDisable(GL_BLEND);

	m_flashlight->Render();

	m_hModel->Render();

	for (int i = 0; i < 8; i++)
	{
		glDisable(light_names[i]);
	}

	if (renderUI)
	{
		UpdateProjection(true);

		m_Minimap->Render();

		if (wireframeMode)
			m_viewWindow->Render();

		m_UIText->Render();

		UpdateProjection(true);

	}

	
	if (width&&height)
		UpdateProjection(false);

	if (drawBuffer == 0)
		SwapBuffers(hdc);// Swap the frame buffers.
}

void Scene3D::RenderReflection(float objectY)
{
	glBindFramebuffer(GL_FRAMEBUFFER, ReflectionbufferName);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	/*
	Excluded until clipping has been deciphered
	GLdouble plane_equation[4] = { 0.0f, -1.0f, 0.0f, objectY - 0.1f };

	glClipPlane(GL_CLIP_PLANE0, plane_equation);
	glEnable(GL_CLIP_PLANE0);
	*/

	m_camera.RenderReflection(objectY);

	float x, y, z;
	glLoadIdentity();// load Identity Matrix

	Vector3 position = m_camera.GetReflectPosition();
	Vector3 lookAt = m_camera.GetReflectLook();
	Vector3 up = m_camera.GetReflectUp();
	Vector3 pl_position = m_camera.GetPosition();

	gluLookAt(position.x, position.y, position.z, lookAt.x, lookAt.y, lookAt.z, up.x, up.y, up.z);

	glDisable(GL_DEPTH_TEST);
	glDisable(GL_LIGHTING);
	glPushMatrix();
	glTranslatef(position.x, position.y, position.z);
	glScalef(10.0f, 10.0f, 10.0f);

	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, m_Skybox->GetTexture());
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	glBegin(GL_TRIANGLES);

	for (int i = 0; i < m_Skybox->GetVertexCount(); ++i)
	{
		VertexType vertex = m_Skybox->GetVertex(i);
		glColor4f(0.5f, 0.5f, 0.5f, 1.0f);
		glTexCoord2f(vertex.tu, vertex.tv);
		glVertex3f(vertex.x, vertex.y, vertex.z);
	}
	glEnd();
	glPopMatrix();

}

void Scene3D::RenderRefraction(float objectY)
{
	glBindFramebuffer(GL_FRAMEBUFFER, RefractionbufferName);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	//GLdouble plane_equation[4] = { 0.0f, -1.0f, 0.0f, objectY + 0.1f };

	//glClipPlane(GL_CLIP_PLANE0, plane_equation);
	//glEnable(GL_CLIP_PLANE0);
	
	glDisable(GL_FOG);

	float x, y, z;
	m_water->GetTranslation(x, y, z);

	glLoadIdentity();// load Identity Matrix

	m_camera.RenderRefraction(Vector3(x, y, z));

	Vector3 position = m_camera.GetRefractPosition();
	Vector3 lookAt = m_camera.GetRefractLook();
	Vector3 up = m_camera.GetRefractUp();


	gluLookAt(x, y, z, lookAt.x, lookAt.y, lookAt.z, up.x, up.y, up.z);

	glPushMatrix();
	m_terrain->GetScaling(x, y, z);
	glScalef(x, y, z);
	m_terrain->GetTranslation(x, y, z);
	glTranslatef(x, y, z);
	m_terrain->GetRotation(x, y, z);
	glRotatef(z, 0, 0, 1);
	glRotatef(y, 0, 1, 0);
	glRotatef(x, 1, 0, 0);

	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, m_terrain->GetTexture());
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	glBegin(GL_TRIANGLES);//Begin drawing state

	if (m_terrain->isIndexed)
	{
		DrawIndexedModel(m_terrain);
	}
	else
	{
		DrawModel(m_terrain);
	}

	glEnd();//end drawing

	glPopMatrix();

	glEnable(GL_FOG);

	//glDisable(GL_CLIP_PLANE0);
}

void Scene3D::BlendTextures(GLuint texA, GLuint texB, float blendFactor)
{
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, texA);
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, texB);
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE);
	
	glTexEnvf(GL_TEXTURE_ENV, GL_COMBINE_RGB, GL_INTERPOLATE);

	glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE0_RGB, GL_PREVIOUS);

	glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE1_RGB, GL_TEXTURE);
	
	glTexEnvf(GL_TEXTURE_ENV, GL_SOURCE2_RGB, GL_CONSTANT);

	glTexEnvf(GL_TEXTURE_ENV, GL_OPERAND2_RGB, GL_SRC_ALPHA);

	//Set the blend factor
	float myColor[4] = { 0.0f, 0.0f, 0.0f, blendFactor };

	glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, myColor);

}

void Scene3D::DrawModel(Model* model)
{
	for (int i = 0; i < model->GetVertexCount(); ++i)
	{
		VertexType vertex = model->GetVertex(i);

		glNormal3f(vertex.nx, vertex.ny, vertex.nz);
		glTexCoord2f(vertex.tu, vertex.tv);
		glVertex3f(vertex.x, vertex.y, vertex.z);
	}

	glEnd();//end drawing

	glPopMatrix();
}

void Scene3D::DrawIndexedModel(Model* model)
{
	Vector3 normalVector;

	for (int i = 0; i < model->GetIndexCount(); ++i)
	{
		VertexType vertex = model->GetVertex(model->GetIndex(i));

		normalVector = model->SampleBumpMap(vertex.tu, vertex.tv + water_translation);
		glNormal3f(vertex.nx, vertex.ny, vertex.nz);
		glMultiTexCoord2f(GL_TEXTURE0, vertex.tu + (normalVector.x *0.01f), vertex.tv + (normalVector.y * 0.01f));
		glMultiTexCoord2f(GL_TEXTURE1, vertex.tu + (normalVector.x * 0.01f), vertex.tv + (normalVector.y * 0.01f));
		glVertex3f(vertex.x, vertex.y, vertex.z);
	}


}

bool Scene3D::InitializeRefractionBuffer()
{
	RefractionbufferName = 0;
	glGenFramebuffers(1, &RefractionbufferName);
	glBindFramebuffer(GL_FRAMEBUFFER, RefractionbufferName);

	glGenTextures(1, &refractionTexture);

	glBindTexture(GL_TEXTURE_2D, refractionTexture);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 512, 512, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

	glGenRenderbuffers(1, &refractiondepthbuffer);
	glBindRenderbuffer(GL_RENDERBUFFER, refractiondepthbuffer);
	glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT, 1024, 1024);
	glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, refractiondepthbuffer);

	glFramebufferTexture(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT1, refractionTexture, 0);

	RefractionDrawBuffers[0] = GL_COLOR_ATTACHMENT1;
	glDrawBuffers(1, RefractionDrawBuffers);

	if (glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
		return false;
}

bool Scene3D::InitializeReflectionBuffer()
{
	ReflectionbufferName = 0;
	glGenFramebuffers(1, &ReflectionbufferName);
	glBindFramebuffer(GL_FRAMEBUFFER, ReflectionbufferName);

	glGenTextures(1, &reflectionTexture);

	glBindTexture(GL_TEXTURE_2D, reflectionTexture);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 512, 512, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);
	
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

	glGenRenderbuffers(1, &depthrenderbuffer);
	glBindRenderbuffer(GL_RENDERBUFFER, depthrenderbuffer);
	glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT, 1024, 1024);
	glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, depthrenderbuffer);

	glFramebufferTexture(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, reflectionTexture, 0);

	ReflectionDrawBuffers[0] = GL_COLOR_ATTACHMENT0;
	glDrawBuffers(1, ReflectionDrawBuffers);

	if (glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
		return false;
}


void Scene3D::SetLights()
{
	int light_num = 0;

	for (int i = 0; i < directionalLights.size(); i++)
	{
		if (light_num < 8)
		{
			if (light_num == 0)
			{
				GLfloat Light_Ambient[] = { ambientLight.r, ambientLight.g, ambientLight.b, 1.0f };
				glLightfv(light_names[light_num], GL_AMBIENT, Light_Ambient);
			}

			GLfloat Light_Diffuse[] = { directionalLights[i].r, directionalLights[i].g, directionalLights[i].b, directionalLights[i].a };
			GLfloat Light_Position[] = { directionalLights[i].dirX, directionalLights[i].dirY, directionalLights[i].dirZ, 0.0f };

			glLightfv(light_names[light_num], GL_DIFFUSE, Light_Diffuse);
			glLightfv(light_names[light_num], GL_POSITION, Light_Position);
			glEnable(light_names[light_num]);
			light_num++;
		}
	}

	for (int i = 0; i < pointLights.size(); i++)
	{
		if (light_num < 8)
		{
			if (light_num == 0)
			{
				GLfloat Light_Ambient[] = { ambientLight.r, ambientLight.g, ambientLight.b, 1.0f };
				glLightfv(light_names[light_num], GL_AMBIENT, Light_Ambient);
			}

			GLfloat Light_Diffuse[] = { pointLights[i].r, pointLights[i].g, pointLights[i].b, pointLights[i].a };
			GLfloat Light_Position[] = { pointLights[i].posX, pointLights[i].posY, pointLights[i].posZ, 1.0f };

			glLightfv(light_names[light_num], GL_DIFFUSE, Light_Diffuse);
			glLightfv(light_names[light_num], GL_POSITION, Light_Position);
			glEnable(light_names[light_num]);
			light_num++;
		}
	}

	for (int i = 0; i < spotLights.size(); i++)
	{
		if (light_num < 8)
		{
			if (light_num == 0)
			{
				GLfloat Light_Ambient[] = { ambientLight.r, ambientLight.g, ambientLight.b, 1.0f };
				glLightfv(light_names[light_num], GL_AMBIENT, Light_Ambient);
			}

			GLfloat Light_Diffuse[] = { spotLights[i].r, spotLights[i].g, spotLights[i].b, spotLights[i].a };
			GLfloat Light_Position[] = { spotLights[i].posX, spotLights[i].posY, spotLights[i].posZ, 1.0f };
			GLfloat Light_Direction[] = { spotLights[i].dirX, spotLights[i].dirY, spotLights[i].dirZ };

			glLightfv(light_names[light_num], GL_DIFFUSE, Light_Diffuse);
			glLightfv(light_names[light_num], GL_POSITION, Light_Position);
			glLightf(light_names[light_num], GL_SPOT_CUTOFF, spotLights[i].spot_cutoff);
			glLightfv(light_names[light_num], GL_SPOT_DIRECTION, Light_Direction);
			glLightf(light_names[light_num], GL_SPOT_EXPONENT, spotLights[i].spot_exponent);
			glEnable(light_names[light_num]);
			light_num++;
		}
	}
}

void Scene3D::Shutdown()
{
	for (int i = 0; i < numObjects; i++)
	{
		m_Objects[i]->Shutdown();
	}
	delete[] m_Objects;
	m_Objects = nullptr;

	delete m_Cube;
	m_Cube = nullptr;
}

void Scene3D::Resize()
{
	if(hwnd == NULL)
		return;

	GetClientRect(*hwnd, &screenRect);	
	ResizeGLWindow(screenRect.right, screenRect.bottom);	

	m_Minimap->SetTranslation(-(screenRect.right / 2) + 110.0f, (screenRect.bottom / 2) - 110.0f, 0.0f);
	m_viewWindow->SetTranslation(0.0f, 0.0f, 0.0f);
	m_UIText->SetTranslation(-(s_width / 2) + 10.0f, (s_height / 2) - 230.0f, 0.0f);
}