#ifndef SCENE3D_H
#define SCENE3D_H

#define _USE_MATH_DEFINES

#include <windows.h>
#include <stdio.h>
#include <mmsystem.h>
#include <math.h>
#include <gl/glew.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <vector>
#include <SOIL.h>
#include <thread>
#include <chrono>
#include <iostream>
#include <sstream>

#include "Input.h"
#include "Model.h"
#include "TerrainModel.h"
#include "ObjModel.h"
#include "PrimitiveModels.h"
#include "SkyboxModel.h"
#include "CameraViewModel.h"
#include "ObjModel.h"
#include "Font.h"
#include "FontModel.h"
#include "Vector3.h"
#include "Camera.h"
#include "H_Model.h"

using std::thread;
using std::chrono::duration_cast;
using std::chrono::milliseconds;
typedef std::chrono::steady_clock the_clock;


#define COLOUR_DEPTH 16	//Colour depth

class Scene3D
{
private:
	struct LightData
	{
		float r, g, b, a;
	};

	struct AmbientLightData : LightData
	{

	};

	struct PointLightData : LightData
	{
		float posX, posY, posZ;
		float const_attenuation;
		float linear_attenuation;
		float quadratic_attenuation;
	};

	struct DirectionalLightData : LightData
	{
		float dirX, dirY, dirZ;
	};

	struct SpotLightData : LightData
	{
		float dirX, dirY, dirZ;
		float posX, posY, posZ;
		float spot_cutoff;
		float spot_exponent;
		float const_attenuation;
		float linear_attenuation;
		float quadratic_attenuation;
	};


public:
	void Init(HWND* hwnd, Input* in);	//initialse function
	bool Update(float dt);
	void Render(CameraClass* camera, GLuint drawBuffer, bool wireframe, bool renderUI, int width, int height);	// render scene
	void Resize();
	void Shutdown();

protected:
	bool CreatePixelFormat(HDC hdc);
	void ResizeGLWindow(int width, int height);	//width and height
	void InitializeOpenGL(int width, int height); // width and height
	void SetLights();
	void DrawModel(Model* model);
	void DrawIndexedModel(Model* model);
	void RenderReflection(float objectY);
	void RenderRefraction(float objectY);
	bool InitializeReflectionBuffer();
	bool InitializeRefractionBuffer();
	void BlendTextures(GLuint texA, GLuint texB, float blendFactor);
	void UpdateProjection(bool toggle);

	//vars
	HWND* hwnd;
	Input* input;
	RECT screenRect;
	HDC	hdc;
	HGLRC hrc;			//hardware RENDERING CONTEXT
	GLuint ReflectionbufferName;
	GLuint reflectionTexture;
	GLuint depthrenderbuffer;
	GLenum ReflectionDrawBuffers[1];
	GLuint RefractionbufferName;
	GLuint refractionTexture;
	GLuint refractiondepthbuffer;
	GLenum RefractionDrawBuffers[1];
	int s_width, s_height;
	Model** m_Objects;
	Model* m_Cube;
	SkyboxModel* m_Skybox;
	CameraViewModel* m_Minimap;
	CameraViewModel* m_viewWindow;
	ObjModel* m_ObjModel;
	Font* m_Font;
	FontModel* m_UIText;
	Model* m_terrain;
	Model* m_water;
	GeodesicSphere* m_sphere;
	CubeSphere* m_cubeSphere;
	Plane* m_shadowPlane;
	int numObjects;
	float rotation;
	float translation;
	float cumulativeDtime;
	float scaling;
	float texture_translation;
	float water_translation;
	AmbientLightData ambientLight;
	std::vector<DirectionalLightData> directionalLights;
	std::vector<PointLightData> pointLights;
	std::vector<SpotLightData> spotLights;
	CameraClass m_camera;
	CameraClass m_stationaryCamera;
	CameraClass m_topDownCamera;
	GLenum light_names[8];
	bool frameEnabled;
	bool wireframeMode = false;
	bool followTerrain = false;
	bool m_stationaryCam = false;
	GLuint m_shadowImposter;
	float spotAngle = 0.0f;
	ObjModel* m_flashlight;
	HModel* m_hModel;
};

#endif