#ifndef _TERRAINMODEL_H_
#define _TERRAINMODEL_H_

#include <thread>
#include <future>
#include <mutex>
#include <iostream>

#include "Model.h"

const int MAX_TRIANGLES = 10000;
const int THREAD_COUNT = 16;

using std::thread;
using std::promise;
using std::mutex;

class TerrainModel : public Model
{
private:
	struct NodeType
	{
		float positionX, positionZ, width;
		int triangleCount;
		VertexType* vertices;
		NodeType* nodes[4];
	};


	struct TriangleThreadArgs
	{
		int start_index, end_index;
		float positionX, positionZ, width;
		int* count;
	};

	struct NodeThreadArgs
	{
		int start_index, end_index;
		float positionX, positionZ, width;
		NodeType* node;
		int* index;
	};

public:
	bool Initialize(char* filename);
	void Render();
	void Shutdown();
	bool GetHeightAtPosition(float x, float z, float& height);
	bool GetNormalAtPosition(float x, float z, Vector3& normal);

private:
	bool LoadHeightMap(char* filename);
	bool LoadRawHeightMap(char* filename);
	void CalculateNormals();
	void InitializeQuadTree();
	void CalculateMeshDimensions(int vertexCount, float& centerX, float& centerZ, float& meshWidth);
	void CreateTreeNode(NodeType* parentNode, float centerX, float centerZ, float width);
	void CalculateNodeThreaded(NodeThreadArgs args);
	void RenderNode(NodeType* node);
	void ReleaseNode(NodeType* node);
	int CountTriangles(float positionX, float positionZ, float width);
	bool IsTriangleContained(int triangleNum, float positionX, float positionZ, float width);
	void IsTriangleContainedThreaded(TriangleThreadArgs args);
	void FindNode(NodeType* node, float x, float z, float& height);
	void FindNodeNormal(NodeType* node, float x, float z, Vector3& normal);
	bool CheckHeightOfTriangle(float x, float z, float& height, float v0[3], float v1[3], float v2[3]);
	void SetTerrainCoordinates();
	bool BuildTerrainModel();

private:
	int m_terrainWidth, m_terrainHeight;
	int m_triangleCount;
	NodeType* m_parentNode;
	VertexType* m_heightMap;
	mutex count_mutex;
};

#endif